<?
/*************************************************
*
*   REST PHP Class. - Use REST Services!
*   Copyright 2006 Zach Smith
*   http://hoeken.ubersonic.org
*   License: GPL
*
*************************************************/

class REST
{
	private $base;
	private $url;

	public static $cacheResults = true;

	public static $cacheLife = 86400;
	
	public function __construct($url)
	{
		//set our base url.
		$this->base = $url;
	}

	public function addParam($key, $val)
	{
		$this->setUrlParam($key, $val);
	}

	public function addParams($vars = array())
	{
		//loop thru and add our params in.
		if (count($vars))
			foreach ($vars AS $key => $val)
				$this->addParam($key, $val);
	}

	public function getUrl()
	{	
		return $this->url;
	}

	public function getLink($text = 'View Response')
	{
		return "<a href=\"" . $this->getUrl() . "\">$text</a>";
	}

	public function request($vars = array())
	{
		//reset to our base.
		$this->url = $this->base;
		
		//add in our search parameters
		$this->addParams($vars);
		
		//get our data...
		$data = $this->getData();
		
		return $data;
	}
	
	private function getData()
	{
		//init.
		$data = '';
		
		//try our cache.
		$data = $this->getCacheData();

		//if its not there, then try remotely.
		if (!$data)
		{
			//get it.
			$data = $this->getRemoteData();

			//save it.
			$this->saveCacheData($data);
		}
			
		//give it back.
		return $data;
	}

	protected function getCacheKey()
	{
		return "Rest:" . get_class($this) . ":" . sha1($this->url);
	}

	protected function getCacheData()
	{
		if (self::$cacheResults)
		{
			//generate a key.
			$key = $this->getCacheKey();
			
			//get it!
			return CacheBot::get($key, self::$cacheLife);
		}
	}
	
	protected function saveCacheData($data)
	{
		if (self::$cacheResults)
		{
			//generate a key.
			$key = $this->getCacheKey();
			
			//get it!
			return CacheBot::set($key, $data, self::$cacheLife);
		}
	}

	private function getRemoteData()
	{
		return $this->curlLoadUrl();
	}

	private function curlLoadUrl()
	{
		$ch = curl_init();
		$timeout = 5; 
		curl_setopt ($ch, CURLOPT_URL, $this->url);
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
		$file_contents = curl_exec($ch);
		curl_close($ch);

		return $file_contents;
	}

	private function setUrlParam($name, $value)
	{
		//temp variable.
		$url = $this->url;
		
		//does it already have any stuff?
		if ($pos = strpos($url, '?'))
		{
			$vars = substr($url, $pos+1);
			$url = substr($url, 0, $pos);
		}

		//get our params...
		$getAr = array();
		$paramAr = explode('&', $vars);
		foreach ($paramAr AS $param)
		{
			if ($param)
			{
				$vars = explode('=', $param);
				$getAr[$vars[0]] = $vars[1];
			}
		}

		//set our new one...
		$getAr[$name] = urlencode($value);
		
		//now format that url....
		$count = 0;
		$url .= "?";
		foreach ($getAr AS $key => $val)
		{
			if ($count)
				$url .= '&';
				
			$url .= "$key=$val";
		
			$count++;
		}

		//save it back.
		$this->url = $url;
	}
}
?>
