<?
/**
* @defgroup Autoload Autoload System
* @brief Autoload is our class that handles autoloading easily.
*
* You can give it file paths to classes, or supply your own autoload
* callback function that will be called if Autoload cant find the class itself.
* @{
*/

/**
* @brief The autoload interface class
*/
class Autoload
{
	/**
	* This is where we store the array of classes => paths
	*/
	private static $classes = null;

	/**
	* this is where we store an array of callback functions
	*/
	private static $functions = null;
	
	/**
	* private constructor... you dont instantiate this class.
	*/
	private function __construct()
	{
	}

	/**
	* this initializes our list of classes and functions.  it is automatically
	* called when you include the class. no need to call.
	*/
	public static function init()
	{
		//only init if its new.
		if (self::$classes === null)
		{
			self::$classes = array();
		
			//basejumper is the main one =)
			self::register('BaseJumper', 'basejumper.inc.php');

			//here are all our modules!!
			self::register('BaseAlert', 'modules/base-alert.inc.php');
			self::register('BaseAlerts', 'modules/base-alerts.inc.php');
			self::register('BaseAdmin', 'modules/base-admin.inc.php');
			self::register('BaseCalendar', 'modules/base-calendar.inc.php');
			self::register('BaseComment', 'modules/base-comment.inc.php');
			self::register('BaseCrons', 'modules/base-crons.inc.php');
			self::register('BaseEvent', 'modules/base-event.inc.php');
			self::register('BaseGallery', 'modules/base-gallery.inc.php');
			self::register('BaseGroup', 'modules/base-group.inc.php');
			self::register('BaseImage', 'modules/base-image.inc.php');
			self::register('BaseInvitation', 'modules/base-invitation.inc.php');
			self::register('BaseInvitations', 'modules/base-invitations.inc.php');
			self::register('BaseInbox', 'modules/base-inbox.inc.php');
			self::register('BaseMain', 'modules/base-main.inc.php');
			self::register('BaseMap', 'modules/base-map.inc.php');
			self::register('BaseMessage', 'modules/base-message.inc.php');
			self::register('BaseModule', 'modules/base-module.inc.php');
			self::register('BaseObject', 'modules/base-object.inc.php');
			self::register('BaseOutbox', 'modules/base-outbox.inc.php');
			self::register('BaseTags', 'modules/base-tags.inc.php');
			self::register('BaseThread', 'modules/base-thread.inc.php');
			self::register('BaseTicket', 'modules/base-ticket.inc.php');
			self::register('BaseUser', 'modules/base-user.inc.php');
			self::register('MyManager', 'modules/my-manager.inc.php');
			self::register('MyModule', 'modules/my-module.inc.php');
			self::register('MyObject', 'modules/my-object.inc.php');
			self::register('ObjectManager', 'modules/object-manager.inc.php');
			
			//our stub modules
			self::register('Ticket', 'modules/stubs/ticket.inc.php');

			//stuff for side boxes..
			self::register('BaseBox', 'boxes/base-box.inc.php');
			self::register('DateBox', 'boxes/date-box.inc.php');
			self::register('CalendarBox', 'boxes/calendar-box.inc.php');
			self::register('HelpBox', 'boxes/help-box.inc.php');
			self::register('LoginBox', 'boxes/login-box.inc.php');

			//stuff for our form class
			self::register('Form', 'form/form.inc.php');
			self::register('SyncForm', 'form/sync.inc.php');
			self::register('FCKeditor', 'FCKeditor/fckeditor.php');

			//our templates..
			self::register('BaseTemplate', 'templates/base-template.inc.php');
			self::register('BlankTemplate', 'templates/blank-template.inc.php');
			self::register('HTMLTemplate', 'templates/html-template.inc.php');
			self::register('JSONTemplate', 'templates/json-template.inc.php');
			self::register('XMLTemplate', 'templates/xml-template.inc.php');

			//our caching classes
			self::register('CacheBot', 'cache.inc.php');
			self::register('EasyCache', 'cache.inc.php');
			self::register('NoCache', 'cache.inc.php');
			self::register('EasyMemCache', 'cache.inc.php');
			self::register('EasyFileCache', 'cache.inc.php');
			self::register('EasyDBCache', 'cache.inc.php');

			//utilities are pretty handy.
			self::register('Config', 'config.inc.php');
			self::register('Img', 'image.inc.php');
			self::register('JSON', 'json.inc.php');
			self::register('Linkify', 'linkify.inc.php');
			self::register('Mail', 'mail.inc.php');
			self::register('SMS', 'sms.inc.php');
			self::register('Site', 'site.inc.php');
			self::register('Time', 'time.inc.php');
			self::register('Util', 'util.inc.php');

			//errors are good(?) too..
			self::register('ErrorHandler', 'error.inc.php');
			self::register('PageError', 'exceptions.inc.php');
			self::register('DbException', 'exceptions.inc.php');

			//our REST classes!
			self::register('REST', 'REST/rest.inc.php');
			self::register('Amazon', 'REST/amazon.inc.php');
			self::register('Geocode', 'REST/geocode.inc.php');
			self::register('Traffic', 'REST/traffic.inc.php');
			self::register('YahooWeather', 'REST/weather.inc.php');

			//our PEAR classes
			self::register("XML_Serializer", "XML/Serializer.php");

			//here we have our YMenu stuff
			self::register("YMenu", "yahoo/menu.php");
			self::register("YMenuItem", "yahoo/menu.php");
		}

		//what about our functions?
		if (self::$functions === null)
		{
			self::$functions = array();

			//add our other loaders.
			self::registerFunction('Form::autoload');
			self::registerFunction('BaseJumper::autoload');
		}
	}

	/**
	* Add a class to the autoload system.
	*
	* @param $class the class name to register
	* @param $path the path to the class.  remember to use absolute or relative to the autoload.inc.php library file.
	*/
	public static function register($class, $path)
	{
		self::$classes[strtolower($class)] = $path;
	}

	/**
	* Add a callback function to the autoload system.
   	*
	* Add your own custom autoload function to the system.  Just pass in the
	* string name and if your class isnt found previous in the list of classes,
	* it will call go through the list of functions and eventually call yours.
	*
	* @param $function - the string name of the function (or static class member)
	*/
	public static function registerFunction($function)
	{
		self::$functions[] = $function;
		self::$functions = array_unique(self::$functions);
	}
	
	/**
	* Dynamically load a class.
	*
	* this function will do the actual loading of the class.  its called from
	* __autoload, so you wont ever need to call this.
	*
	* @param $class the name of the class to load.
	*/
	public static function load($class)
	{
		//if its in our list of classes, include it.
		if (self::$classes[strtolower($class)])
			require_once(self::$classes[strtolower($class)]);
		//nope, maybe one of our other autoload guys has it.
		else if (count(self::$functions))
		{
			//loop thru all our functions.
			foreach (self::$functions AS $function)
			{
				//look until we find it.
				if (!class_exists($class, false))
					eval("$function('$class');");
			}
		}
	}
}

//define our autoload function
if (!function_exists('__autoload'))
{
	/**
	* Magic function to load include files as needed
	* @param $class Class name for which include is needed
	*/
	function __autoload($class)
	{
		Autoload::load($class);
	}
}
//oops, autoload already defined... throw an error
else
{
	throw new Exception("__autoload() already defined.  If you want BaseJumper to
	work, register your autoload function with Autoload:registerFunction() and
	use the built-in __autoload()");
}
/*@}*/

//initialize our autoload system.
Autoload::init();
?>
