<?php
/**
* A button
* @ingroup BasicFields
* @author Scott Peshak
*/
class ButtonField extends AbstractField
{
	protected $type = 'button'; ///< Button type, one of 'reset', 'submit', 'button'

	/**
	 * Constructor
	 * @param $name Name of button
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs array of extra attributes.
	 */
	public function __construct($name, &$parentForm, array $attribs = array('type' => 'button'))
	{
		parent::__construct($name, $parentForm, $attribs); 
	}
	
	/**
	 * Draw the button
	 */
	public function drawInput()
	{
		$extra = $this->extrasAsAttributes();
		echo "<input type='$this->type' value='$this->value' $extra/>";
	}

	/**
	 * Magic setter
	 * @param $name Name of the member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		if ($type == 'type' && !preg_match('/(submit)|(reset)|(button)/', $value))
			throw new InvalidAttributeValueException($value);
		
		parent::__set($name, $value);
	}

	####
	# All of the following are effective no-ops just to fully implement Field
	####
	/**
	 * Validate
	 * Needed to fully implement Field, but nothing is done for a button
	 */
	public function validate()
	{
		return; #No-Op
	}

	/**
	 * Get data from field
	 * Needed to fully implement Field, but nothing is done for a button
	 */
	public function getData()
	{
		return; #No-Op
	}

	/**
	 * Get SQL string 
	 * Needed to fully implement Field, but nothing is done for a button
	 */
	public function getSqlImpl()
	{
		return null; #No-Op
	}

	/**
	 * Check for errors
	 * @return Boolean \c true because buttons can't have errors
	 */
	public function hasError()
	{
		return false; # Buttons are perfect and never have errors
	}

}
?>
