<?php
/**
* Field that is active only when an accompaning checkbox is selected (or not selected)
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-05-29 14:55:45 -0500 (Mon, 29 May 2006) $
*/
class ConditionalField extends CollectionField
{
	private $activeOn; ///< What condition the field will be active on
	protected static $jsDrawn = false; //< If the JavaScript has been drawn yet or not

	/**
	 * Constructor
	 * @param $name Name of collection
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Extra attributes
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array()) 
	{
		if(array_key_exists('fields', $attribs) === false)
			throw new Exception("key 'fields' must exist in attribs array");

		if(count($attribs['fields']) != 1)
			throw new Exception("only one field can be added to a ConditionalField");

		array_unshift($attribs['fields'], array('CheckboxField', "${name}_box", $attribs));

		# Set a "normal" default
		$this->activeOn = true;
		parent::__construct($name, $parentForm, $attribs);
	}


	/**
	 * Magic setter
	 * @param $name Name of member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		if($name == 'activeOn')
			$this->activeOn = $value;
		else
			parent::__set($name, $value);
	}
			
	
	/**
	 * Draw the HTML for the field
	 */
	public function drawInput()
	{
		$subFieldArr = $this->subFieldsToJsArray();
		$this->fields[0]->onchange = "changeCondState(this, $subFieldArr, " . ($this->activeOn ? 'true' : 'false') . ');';
		parent::drawInput();
	}
	
	/**
	 * Get the JavaScript for the conditional field.
	 * @return string of JavaScript
	 */
	public function getJs()
	{
		if(self::$jsDrawn)
			return null;

		self::$jsDrawn = true;

		return <<<EOJAVASCRIPT
		function changeCondState(chkbox, fields, activeOn)
		{
			var newDisabled;

			if(chkbox.checked == activeOn)
				newDisabled = false;
			else
				newDisabled = true;


			for(var i = 0; i < fields.length; i++)
			{
				var fieldObj = document.getElementById(fields[i]);
				fieldObj.disabled = newDisabled;
			}
		}
EOJAVASCRIPT;
	}

	
	/**
	 * Get the list of sub fields (except the checkbox) as a javascript array
	 * @return String of javascript ready for use
	 */
	protected function subFieldsToJsArray()
	{
		$fieldNames = array();
		foreach($this->fields as $field)
		{
			$fieldNames[] = $field->name;
		}
		array_shift($fieldNames); # don't include the checkbox (which should be the first field)

		return 'new Array(\'' . implode('\', \'', $fieldNames) . '\')';
	}
}
?>
