<?php
/**
* Combination date & time field
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-04-10 12:21:54 -0500 (Mon, 10 Apr 2006) $
*/
class DateTimeField extends CollectionField
{
	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array('mode' => 12))
	{
		$attribs['fields'] = array(
			array('DateField', "${name}_date", $attribs),
			array('TimeField', "${name}_time", $attribs)
		);

		parent::__construct($name, $parentForm, $attribs);
	}

	/**
	 * Set the field data
	 * @param $value Value to set, should be a full datetime string
	 */
	public function setData($value)
	{
		$this->fields[0]->setData($value);
		$this->fields[1]->setData($value);
	}
	
	/**
	 * Get the data
	 * @return Date string
	 */
	public function getData()
	{
		return $this->fields[0]->getData() . ' ' . $this->fields[1]->getData();
	}

	/**
	 * Get SQL string of the field data
	 * @return SQL string ready for use in insert statement
	 */
	public function getSqlImpl()
	{
		return $this->name . '=\'' . $this->getData() . '\'';
	}
}
?>
