<?php
/**
* Select field that gets its values from a database field.
* @ingroup ListFields
* @author Scott Peshak
* @date $Date: 2006-04-11 03:04:09 -0500 (Tue, 11 Apr 2006) $
*/
class DbSelectField extends SelectField
{
	/**
	 * Constructor
	 * @param $name Name of input
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes
	 */
	function __construct($name, &$parentForm, array $attribs = array())
	{
		if(!array_key_exists('query', $attribs))
			throw new Exception('Query not set');

		$query = $attribs['query'];
		unset($attribs['query']);

		$attribs['options'] = array();
		parent::__construct($name, $parentForm, $attribs);
		$this->options = $this->queryToValues($query);
	}

	/**
	 * Run a query and create an array to pass to the SelectField constructor
	 * @param $query Query to run
	 */
	function queryToValues($query)
	{
		$dbConn = $this->parentForm->dbConn;
		$res =& $dbConn->query($query);
		$ret = array();

		while($row = $res->fetchRow(DB_FETCHMODE_ASSOC))
		{
			if(!array_key_exists('value', $row) || !array_key_exists('title', $row))
				throw new InvalidQueryException("Query did not include columns 'value' & 'title'");

			$ret[$row['value']] = $row['title'];
		}

		return $ret;
	}
}
?>
