<?php
/**
* Field interface.
* All fields must implement this, although most will just inheirt from AbstractField so this 
* really is just a formality
* @ingroup BasicFields
* @author Scott Peshak
* @date $Date: 2006-05-19 21:27:22 -0500 (Fri, 19 May 2006) $
*/
interface Field
{
	/**
	 * Constructor
	 * @param $name Field name
	 * @param $parentForm Form object that owns the field.
	 * @param $attribs Array of extra attributes
	 */
	public function __construct($name, Form &$parentForm, array $attribs);

	/**
	 * Draw the HTML input for the field
	 */
	public function drawInput();

	/**
	 * Get the value of the Field 
	 * @return Value of the Field
	 */
	public function getData();

	/**
	 * Validate the input of the form.
	 * This sets the form data from the $_REQUEST input, and runs any
	 * sort of validation routines needed
	 */
	public function validate();

	/**
	 * Get an SQL representation of the field data
	 * @return SQL string ready to use in a query.
	 */
	public function getSqlImpl();

	/**
	 * Check to see if the field has any errors
	 * @return Boolean \c true if the field has an error, \c false otherwise
	 */
	public function hasError();

	/**
	 * Set the field data to a passed value
	 * @param $value Value to set
	 */
	public function setData($value);

	/**
	 * Get any JavaScript that is needed by this field.
	 * @return \c string of JavaScript
	 */
	public function getJs();

	/**
	 * Get any special CSS that is needed by the field.
	 * @return \c string of CSS
	 */
	public function getCss();

	/**
	 * Force a field to have an error.
	 */
	public function forceError();
}
?>
