<?php
/**
* Field to display a Internet Protocol (IP) address
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-05-07 22:09:30 -0500 (Sun, 07 May 2006) $
*/
class IpAddressField extends CollectionField
{
	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		$attribs['fields'] = array(
			array('TextField', "${name}_1", array('size' => 3, 'maxlength' => 3)),
			array('LabelField', "${name}_dot1", array('text' => '&nbsp;.&nbsp;')),
			array('TextField', "${name}_2", array('size' => 3, 'maxlength' => 3)),
			array('LabelField', "${name}_dot2", array('text' => '&nbsp;.&nbsp;')),
			array('TextField', "${name}_3", array('size' => 3, 'maxlength' => 3)),
			array('LabelField', "${name}_dot3", array('text' => '&nbsp;.&nbsp;')),
			array('TextField', "${name}_4", array('size' => 3, 'maxlength' => 3)),
		);

		parent::__construct($name, $parentForm, $attribs);
		$this->autoAdvance = true;
	}

	/**
	 * Check the value for validity
	 * @return Boolean \c true if the value seems to be a valid IP, \c false otherwise
	 */
	public function validate()
	{
		# call parent to validate individual text fields
		parent::validate();

		# If any of the sub fields have input automatically all sub fields _must_ have input
		if(!$this->allFieldsHaveData() && !$this->noFieldsHaveData())
		{
			$this->hasError = true;
			$this->errorMessage = "The $this->title field must be a valid IP address.";
		}
	}

	/**
	 * Get the data for the field
	 * @return Current data
	 */
	public function getData()
	{
		$ret = $this->fields[0]->getData() . '.' . $this->fields[2]->getData() . '.' . 
			$this->fields[4]->getData() . '.' . $this->fields[6]->getData();

		# check for an empty value
		if($ret == '...')
			return null;

		return $ret;
	}


	/**
	 * Set the data for the field
	 * @param $data Value to set
	 */
	public function setData($data)
	{
		if(preg_match('/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/', $data, $matches))
		{
			if(($matches[1] <= 255 && $matches[1] >= 0) &&
				($matches[2] <= 255 && $matches[2] >= 0) &&
				($matches[3] <= 255 && $matches[3] >= 0) &&
				($matches[4] <= 255 && $matches[4] >= 0)
			)
			{
				$this->fields[0]->setData($matches[1]);
				$this->fields[2]->setData($matches[2]);
				$this->fields[4]->setData($matches[3]);
				$this->fields[6]->setData($matches[4]);

				return;
			}
		}

		if($data != '')
			throw new InvalidValueException();
	}

	/**
	 * Get SQL string
	 */
	public function getSqlImpl()
	{
		return "$this->name = '" . $this->getData() . "'";
	}
}
?>
