<?php
/**
 * Password change field
 * @ingroup CollectionFields
 * @author Scott Peshak
 * @date $Date: 2006-04-18 00:59:51 -0500 (Tue, 18 Apr 2006) $
 *
 * Use when a user is changing/setting a password, prompts the user twice and ensures that the same value was entered.
 *
 * Note: This fields doesnt work very well *yet*
 *
 * When making use of this field you will want to add the following (or
 * something like it) to your site's CSS to make the password strength meter
 * work correctly.
 * 
 * @code
 *	*.pwdMeterActive, *.pwdMeterInactive 
 *	{
 *		border: 1px solid black;
 *		height: 5px;
 *		width: 25px;
 *		margin: 0px 0px 3px 3px;
 *		float: left;
 *	}
 *
 *	*.pwdMeterActive
 *	{
 *		background-color: green;
 *	}
 *
 *	*.pwdMeterInactive
 *	{
 *		background-color: grey;
 *	}
 *
 *	*.pwdMeterLabel
 *  {
 *		font-size: 0.8em;
 *	} 
 * @endcode
 */
class PasswordChangeField extends CollectionField
{
	static protected $meterJsDrawn = false;
	static protected $suggestJsDrawn = false;
	static protected $cssDrawn = false;
	protected $meter;
	protected $suggest;

	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		$attribs['fields'] = array(
			array('PasswordField', "${name}_1", $attribs),
			array('PasswordField', "${name}_2", $attribs)
		);

		$attribs['arrangement'] = 'vertical';

		$this->meter = true;
		$this->suggest = false;
		parent::__construct($name, $parentForm, $attribs);
	}

	
	/**
	 * Magic setter
	 * @param $name Name of member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		if($name == 'meter')
			$this->meter = $value;
		elseif($name == 'suggest')
			$this->suggest = $value;
		else
			parent::__set($name, $value);
	}

	/**
	 * Draw the input
	 */
	public function drawInput()
	{
		$this->fields[0]->onkeyup = "updatePasswordMeter('$this->name', testPasswordStrength(this.value))";
		parent::drawInput();

		if($this->suggest)
		{
			$passwords = null;
			exec('apg -n 5', $passwords);
			
			echo '<div class="pwdSuggest">';
			echo '<div style="border-bottom: 1px solid gray;">Password Suggestions</div>';

			foreach($passwords as $passwd)
				echo "<span onclick=\"useSuggestion('$this->name', this);\" class='passwd'>$passwd</span><br/>";

			echo '</div>';
		}

		if($this->meter)
		{
			echo <<<EOHTML
			<span id='{$this->name}_meterlabel' class='pwdMeterLabel'>Password Quality:</span>
			<div>	
				<div id='{$this->name}_meter1' class='pwdMeterInactive'></div>
				<div id='{$this->name}_meter2' class='pwdMeterInactive'></div>
				<div id='{$this->name}_meter3' class='pwdMeterInactive'></div>
				<div id='{$this->name}_meter4' class='pwdMeterInactive'></div>
				<div style="clear:both;"></div>
			</div>
EOHTML;
		}
	}

	
	/**
	 * Validate the input.
	 * This makes sure that the user input the same value twice
	 */
	public function validate()
	{
		parent::validate();
		
		if($this->fields[0]->getData() != $this->fields[1]->getData())
		{
			$this->hasError = true;
			$this->errorMessage = "The passwords entered do not match";

			# clear the 2nd password field, and mark it as error'ed
			$this->fields[1]->value = null;
			$this->fields[1]->forceError();
		}
	}

	/**
	 * Get the value of the field
	 * @return User entered data
	 */
	public function getData()
	{
		return $this->fields[0]->getData();
	}

	/**
	 * Get an SQL string for the form
	 * @return SQL string ready for use in a query
	 */
	public function getSqlImpl()
	{
		# Use the first PasswordField's getSqlImpl, changing the column name to this field's name
		return str_replace("{$this->name}_1", $this->name, $this->fields[0]->getSqlImpl());
	}

	/**
	 * Get the JavaScript required by this field
	 */
	public function getJs()
	{
		$js = null;

		if(!self::$meterJsDrawn && $this->meter)
		{
			self::$meterJsDrawn = true;
			$js = file_get_contents('inc/form/js/passwordchange-meter.js', true); # this is going to cause troubles...
		}

		if(!self::$suggestJsDrawn && $this->suggest)
		{
			self::$suggestJsDrawn = true;
			$js .= file_get_contents('inc/form/js/passwordchange-suggest.js', true);
		}

		return $js;
	}


	/**
	 * Get the special CSS for this field.
	 * @return \c string of CSS
	 */
	public function getCss()
	{
		if(self::$cssDrawn)
			return null;

		self::$cssDrawn = true;
		return <<<EOCSS
			*.pwdMeterActive, *.pwdMeterInactive 
			{
				border: 1px solid black;
				height: 5px;
				width: 25px;
				margin: 0px 0px 3px 3px;
				float: left;
			}

			*.pwdMeterActive
			{
				background-color: green;
			}

			*.pwdMeterInactive
			{
				background-color: grey;
			}

			*.pwdMeterLabel
			{
				font-size: 0.8em;
			} 

			*.pwdSuggest
			{
				border: 1px solid gray;
				background-color: #FFFFCC;
				width: 10em;
				font-size: 0.7em;
			}
EOCSS;
	}
}
?>
