<?php
/**
* Field to display a phone number.
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-04-10 12:13:29 -0500 (Mon, 10 Apr 2006) $
*/
class PhoneNumberField extends CollectionField
{
	protected $ext = false; ///< Boolean flag indicating if an extention field should be used. 

	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		$attribs['fields'] = array(
			array('TextField', "${name}_area", array_merge($attribs, array('size' => '3', 'regExp' => 'number', 'maxlength' => '3'))),
			array('TextField', "${name}_prefix", array_merge($attribs, array('size' => '3', 'regExp' => 'number', 'maxlength' => '3'))),
			array('TextField', "${name}_num", array_merge($attribs, array('size' => '4', 'regExp' => 'number', 'maxlength' => '4')))
		);
		parent::__construct($name, $parentForm, $attribs);
		$this->autoAdvance = true;

		if($this->ext)
		{
			$this->add('LabelField', 'Ext.');
			$this->add('TextField', "${name}_ext", array_merge($attribs, array('size' => '4', 'regExp' => 'number')));
		}
	}

	/**
	 * Set the field data to the passed value.
	 * @param $value Value to set, should be a properly formated phone number (IE: 555-123-4567, (555)123-4567)
	 */
	public function setData($value)
	{
		$value = preg_replace("/[^0-9]*/", "", $value);
		
		if(preg_match('/([0-9]{3})([0-9]{3})([0-9]{4})/', $value, $matches))
		{
			$this->fields[0]->setData($matches[1]);
			$this->fields[1]->setData($matches[2]);
			$this->fields[2]->setData($matches[3]);
		}
	}

	/**
	 * Get the data from this field
	 * @return Current value of the field as a normalized phone number (IE: (555)123-4567 x890 )
	 */
	public function getData()
	{
		$num = $this->fields[0]->getData() . $this->fields[1]->getData() . $this->fields[2]->getData();

		if($this->ext)
			$num .= 'x' . $this->fields[4]->getData();

		return $num;
	}

	/**
	 * Get SQL string of the data of this form
	 * @return SQL string
	 */
	public function getSqlImpl()
	{
		return "$this->name = '" . $this->getData() . "'";
	}

}
?>
