<?php
/**
* Field to display a US Social Security Number.
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-04-11 01:33:17 -0500 (Tue, 11 Apr 2006) $
*/
class SSNField extends CollectionField
{
	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form object that the field belongs to.
	 * @param $attribs Array of extra attributes.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		$attribs['fields'] = array(
			array('TextField', "${name}_1", array('size' => 3, 'regExp' => '/^[0-9]{3}$/', 'maxlength' => 3)),
			array('LabelField', "${name}_dash1", array('text' => '-')),
			array('TextField', "${name}_2", array('size' => 2, 'regExp' => '/^[0-9]{2}$/', 'maxlength' => 2)),
			array('LabelField', "${name}_dash2", array('text' => '-')),
			array('TextField', "${name}_3", array('size' => 4, 'regExp' => '/^[0-9]{4}$/', 'maxlength' => 4))
		);

		parent::__construct($name, $parentForm, $attribs);

		$this->autoAdvance = true;
	}

	/**
	 * Check the value for validity
	 * @return Boolean \c true if the value seems to be a valid SSN, \c false otherwise
	 */
	public function validate()
	{
		# call parent to validate individual text fields
		parent::validate();

		# If any of the sub fields have input automatically all sub fields _must_ have input
		if(!$this->allFieldsHaveData() && !$this->noFieldsHaveData())
		{
			$this->hasError = true;
			$this->errorMessage = "The $this->title field must be a valid SSN number.";
		}
	}


	/**
	 * Get the data for this field
	 * @return Current data
	 */
	public function getData()
	{
		$ret = $this->fields[0]->getData() . '-' . $this->fields[2]->getData() . '-' . $this->fields[4]->getData();
		
		# Check for empty values
		if($ret == '--')
			return null;

		return $ret;
	}


	/**
	 * Set the data for the field
	 * @param $data Value to set
	 */
	public function setData($data)
	{
		# This nasty huge regex matches 123-45-6789, 123 45 6789, and 123456789
		if(preg_match('/(^|\s)(00[1-9]|0[1-9]0|0[1-9][1-9]|[1-6]\d{2}|7[0-6]\d|77[0-2])(-?|[\. ])([1-9]0|0[1-9]|[1-9][1-9])\3(\d{3}[1-9]|[1-9]\d{3}|\d[1-9]\d{2}|\d{2}[1-9]\d)($|\s|[;:,!\.\?])/', $data, $matches))
		{
			$this->fields[0]->setData($matches[2]);
			$this->fields[2]->setData($matches[4]);
			$this->fields[4]->setData($matches[5]);
		}
		elseif($data != '')
			throw new InvalidValueException();
	}

	/**
	 * Get SQL string
	 */
	public function getSqlImpl()
	{
		return "$this->name = '" . $this->getData() . "'";
	}
}
?>
