<?php
/**
* Time field
* @ingroup CollectionFields
* @author Scott Peshak
* @date $Date: 2006-04-09 23:04:29 -0500 (Sun, 09 Apr 2006) $
* @todo make this an actual collection field
*/
class TimeField extends AbstractField
{
	/**
	 * Flag to determine if the field should be 24 or 12 hour 
	 */
	protected $mode = '12';

	/**
	 * Constructor
	 * @param $name Name of field
	 * @param $parentForm Form that owns the field.
	 * @param $attribs Array of extra attributes.
	 * Special Keys:
	 *	- \c mode Either \c 12 or \c 24, defined if a 12 or 24 hour format should be used.
	 */
	public function __construct($name, Form &$parentForm, array $attribs = array())
	{
		parent::__construct($name, $parentForm, $attribs);

		if(isset($this->extras['mode']))
		{
			if($this->extras['mode'] == 12 || $this->extras['mode'] == 24) 
				$this->mode = $this->extras['mode'];
			else
				throw new Exception("Invalid time field mode: $this->extras[mode]");

			unset($this->extras['mode']);
		}
		else
			$this->mode = 12;

		$this->value = 'now';
	}

	
	/**
	 * Draw the field
	 */
	public function drawInput()
	{
		$time = getdate(strtotime($this->value));
		
		# Hour
		$displayHour = $this->mode == 12 ? ($time['hours'] > 12 ? $time['hours'] - 12 : $time['hours']) 
			: $time['hours'];

		echo "<select name='{$this->name}_hour'>";
		for($i = 1; $i <= $this->mode; $i++)
			echo "\t<option value='$i'", $displayHour == $i ? ' selected="selected">' : '>',
				$i, "</option>\n";
		echo "</select>:";

		# Minute
		echo "<select name='{$this->name}_minute'>";
		for($i = 0; $i <= 60; $i++)
			echo "<option value='$i'", $time['minutes'] == $i ? ' selected="selected">' : '>',
				str_pad($i, 2, 0, STR_PAD_LEFT), "</option>\n";

		echo "</select>";

		# Meridiem
		if($this->mode == 12)
		{
			echo "\n<select name='{$this->name}_meridiem'>";
			echo '<option value="am"' . ($time['hours'] <= 12 ? ' selected="selected">' : '>') . 'AM</option>';
			echo '<option value="pm"' . ($time['hours'] > 12 ? ' selected="selected">' : '>') . 'PM</option>';
			echo "</select>\n";
		}
	}

	/**
	 * Set the field data from the $_REQUEST array
	 */
	public function setDataFromRequest()
	{
		if(isset($_REQUEST["{$this->name}_meridiem"]))
			$hour = $_REQUEST["{$this->name}_meridiem"] == 'pm' ? $_REQUEST["{$this->name}_hour"] + 12 :
				(int)$_REQUEST["{$this->name}_hour"];
		else
			$hour = (int)$_REQUEST["{$this->name}_hour"];
			
		$this->value = "$hour:" . str_pad($_REQUEST["{$this->name}_minute"], 2, 0);
	}
}
?>
