<?php
/**
* File upload field
* @ingroup BasicFields
* @author Scott Peshak
* @date $Date: 2006-04-10 02:19:56 -0500 (Mon, 10 Apr 2006) $
*/
class UploadField extends AbstractField
{
	protected $dir = ''; ///< Path to upload into

	/**
	 * Magic setter
	 * @param $name Name of the member to set
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		if($name == 'dir')
			$this->dir = $value;
		else
			parent::__set($name, $value);
	}

	/**
	 * Draw the input
	 */
	public function drawInput()
	{
		$maxSize = min($this->returnBytes(ini_get('upload_max_filesize')), $this->returnBytes(ini_get('post_max_size')));
		echo <<<EOHTML
		<input type="file" name="$this->name" size="30">
		<input type="hidden" name="MAX_FILE_SIZE" value="$maxSize"/>
EOHTML;
	}


	/**
	 * Validate the field.
	 * This handles moving the file to the upload directory.
	 */
	public function validate()
	{
		//was there an error?
		if($_FILES[$this->name]['error'])
		{
			if($_FILES[$this->name]['error'] == 4 && $this->required)
			{
				$this->hasError = true;
				$this->errorMessage = "There was an error with the $this->title field.";
				return;
			}
		}
		
		//is it required?
		if($this->required && !$_FILES[$this->name])
		{
			$this->errorMessage = "The $this->title field is required.";
			$this->hasError = true;
			return;
		}

		//set our data.
		$this->setFromRequest();

		//check our directory stuff.
		if(!$this->dir)
			throw new Exception("UploadField destination directory is not set (Pass the 'dir' option when you add the field).");
		elseif(!is_writable($this->dir))
			throw new Exception("UploadField directory ($this->dir) is not writable.");
			/** @todo display who the server is running as to give a fix hint */

		//finally move it!
		move_uploaded_file($_FILES[$this->name]['tmp_name'], $this->dir . '/' . $_FILES[$this->name]['name']);

		$this->hasError = false;
	}


	/**
	 * Set the value of this field from the $_REQUEST (and $_FILES) array
	 */
	public function setFromRequest()
	{
		$this->value = $_FILES[$this->name]['name'];
	}


	/**
	 * Convert php.ini filesize shorthand into number of bytes.
	 * Lifted from the PHP manual (lookup ini_get)
	 * @param $val Value in shorthand
	 * @return \c int value of the number of bytes
	 */
	protected function returnBytes($val)
	{
	   $val = trim($val);
	   $last = strtolower($val{strlen($val)-1});

	   switch($last)
	   {
		   // The 'G' modifier is available since PHP 5.1.0
		   case 'g':
			   $val *= 1024;
		   case 'm':
			   $val *= 1024;
		   case 'k':
			   $val *= 1024;
	   }

	   return $val;
	}
}
?>
