<?php
/**
* @defgroup FormLayout Form Layouts
* @ingroup Form
* Layouts are different ways to display a form.  One line, one column, two
* columns, etc.  More freedom for you.
*/

/**
 * Layout interface.
 * All layouts must implement this.
 * \ingroup form
 * @author Scott Peshak
 * @date $Date: 2006-03-22 21:18:53 -0600 (Wed, 22 Mar 2006) $
 */
interface Layout
{
	/**
	 * Draw any needed front matter for the layout
	 */
	public function drawBeginning();

	/**
	 * Draw any needed ending output for the layout.
	 */
	public function drawEnd();

	/**
	 * Draw a field.
	 * @param $field Field to draw
	 */
	public function drawField($field);
}

/**
* BaseLayout is a basic layout that implements some nice default behavior for
* you to use.
*
* @ingroup FormLayout
*/
abstract class BaseLayout implements Layout
{
	/**
	 * Array to buffer hidden fields to as they are encountered
	 * This allows the hidden fields to be drawn outside the table so things stay xhtml compliant w/o messing
	 * with visual layout.
	 */
	public $hiddenFields = array();

	/**
	* draw the hidden fields in a nice manner.  Good helper function
	*/
	public function drawHiddenFields()
	{
		if (count($this->hiddenFields))
		{
			foreach($this->hiddenFields as $field)
			{
				echo "\t";
				$field->drawInput();
				echo "\n";
			}
		}
	}
}
?>
