<?php
/**
* Draw the form as a table with one column.
*
* The format is field name, field, field name, field
* 
* @ingroup FormLayout
* @author Zach Smith
*/
class OneColumnTableLayout extends BaseLayout
{
	/**
	* Draw the beginning of the table
	*/
	public function drawBeginning()
	{
		echo '<table class="BaseForm">';
	}

	/**
	* Draw the ending of the table, this flushes out any hidden fields that were encountered
	*/
	public function drawEnd()
	{
		$this->drawHiddenFields();

		echo '</table>';
	}

	/**
	* Draw a field as a table row
	* @param $field Field to draw
	*/
	public function drawField($field)
	{
		if($field instanceof HiddenField)
		{
			$this->hiddenFields[] = $field;
			return;
		}

		//dont draw the title if its a button...
		if (!($field instanceOf ButtonField) && !($field instanceOf StaticField))
		{
			$title = $field->getTitle();
			if ($title)
				echo "<tr><th>$title</th></tr>";
		}

		echo "<tr><td>";
		if($js = $field->getJs())
			echo "<script type='text/javascript'>$js</script>";
		
		$field->drawInput();

		echo "</td></tr>";
		try
		{
			if($field->hasError())
				echo "<tr><td><span class='FormError'>$field->errorMessage</span></td></tr>";
		}
		catch (NotValidatedException $e){};
	}
}
?>
