<?php
/**
* Draw the entire form on one single line.
*
* Good for search forms, or other short forms.
*
* @ingroup FormLayout
* @author Zach Smith
* @date $Date: 2006-07-17 00:57:46 -0600 (Tue, 28 Mar 2006) $
*/
class OneLineLayout extends BaseLayout
{
	/**
	* queue our errors for display after our form is done
	*/
	public $errors = array();
	
	/**
	* draw the beginning of the form.
	*
	* the form is wrapped in a div with class of OneLineForm
	*/
	public function drawBeginning()
	{ 
		echo '<div class="OneLineForm">';
	}

	/**
	* draw the end of the form
	*
	* this will draw the end of the div, then the hidden fields and then the errors
	*/
	public function drawEnd()
	{
		echo '</div>';

		$this->drawHiddenFields();
		$this->drawErrors();
	}
	
	/**
	* Draw a field as a table row
	* @param $field Field to draw
	*/
	public function drawField($field)
	{
		if($field instanceof HiddenField)
		{
			$this->hiddenFields[] = $field;
			return;
		}

		//dont draw the title if its a button...
		if (!($field instanceOf ButtonField) && !($field instanceOf StaticField))
			echo "<span class=\"OneLineTitle\">" . $field->getTitle() . "</span>";

		if($js = $field->getJs())
			echo "<script type='text/javascript'>$js</script>";

		echo '<span class="OneLineField">';
		$field->drawInput();
		echo '</span>';

		try
		{
			if($field->hasError())
				$this->errors[] = "<span class='FormError'>$field->errorMessage</span>";
		}
		catch (NotValidatedException $e){};
	}

	/**
	* draw our errors of our form
	*
	* they will each be wrapped in a div of class "OneLineFormErrors"
	*/
	public function drawErrors()
	{
		if (count($this->errors))
		{
			echo '<div class="OneLineFormErrors">';
			echo implode("<br/>", $this->errors);
			echo '</div>';
		}
	}
}
?>
