<?php
/**
* Draw the form as a two column table.
*
* This is generally the most frequently used form layout and one of the most useful.
* 
* @ingroup FormLayout
* @author Scott Peshak, Zach Smith
*/
class TableLayout extends BaseLayout
{
	/**
	* Draw the beginning of the table
	*
	* The table has a class of 'BaseForm'
	*/
	public function drawBeginning()
	{
		echo "\n\n<table class=\"BaseForm\">\n";
	}

	/**
	* Draw the ending of the table, this draws any hidden fields that were encountered
	*/
	public function drawEnd()
	{
		echo "</table>\n\n";
		
		$this->drawHiddenFields();
	}

	/**
	* Draw a field as a table row
	* @param $field Field to draw
	*/
	public function drawField($field)
	{
		if($field instanceof HiddenField)
		{
			$this->hiddenFields[] = $field;
			return;
		}

		echo "\t<tr>\n";

		if ($field instanceOf StaticField)
			echo "\t\t<td colspan=\"2\">";
		else if($field->title !== null && !($field instanceOf ButtonField))
			echo "\t\t<th>" . $field->getTitle() . "</th>\n\t\t<td>";
		else
			echo "\t\t<td>&nbsp;</td>\n\t\t<td>";
		
		if($js = $field->getJs())
			echo "\n\t\t\t<script type='text/javascript'>$js</script>";
		
		$field->drawInput();
		
		echo "</td>\n\t</tr>\n";
		try
		{
			if($field->hasError())
				echo "\n\t<tr><td>&nbsp;</td><td><span class='FormError'>$field->errorMessage</span></td></tr>\n";
		}
		catch (NotValidatedException $e){};
	}
}
?>
