<?
require_once("form.inc.php");

class FormSync extends Form
{
	private $table; ///< Table name in the database that this form is acting upon.
	private $id; ///< ID in the table to use.
	private $dbConn; ///< Database connection object.
	
	public function __construct($layout = null)
	{
		parent::__construct($layout);

		if(isset($GLOBALS['dbconn']))
		{
			self::assertIsDbObject($GLOBALS['dbconn']);
			$this->dbConn = &$GLOBALS['dbconn'];
		}
		
		$this->id = 0;
	}
	
	/**
	 * Magic setter
	 * @param $name Name of member
	 * @param $value Value to set
	 */
	public function __set($name, $value)
	{
		switch($name)
		{
			case 'table':
			case 'id':
				$this->$name = $value;
				return true;
				break;

			case 'dbConn':
				self::assertIsDbObject($value);
				$this->dbConn = $value;
				break;

			default:
				return parent::__set($name, $value);
		}
	}
	
	/**
	 * Handle the form submittal
	 * @param $redir URL to redirect to on successful submitting.
	 */
	public function handleSubmit($redir = null)
	{
		if($this->isSubmitted())
		{
			$this->validate();
			if(!$this->hasError())
			{
				$this->sync();
				
				if($redir)
					header("Location: $redir");
			}
		}
	}

	
	/**
	 * Handle the form submittal by calling a callback function.
	 * @param $callback Name of callback function.  Will be passed the output of getData
	 * @param $redir URL to redirect to on successful handling (only redirected to if the callback function returns true)
	 */
	public function handleSubmitWithCallback($callback, $redir = null)
	{
		if($this->isSubmitted())
		{
			$this->validate();

			if(!$this->hasError())
			{
				if(!is_callable($callback))
					throw new InvalidCallbackException();

				$ret = call_user_func($callback, $this->getData(), &$this); 

				if($ret && $redir)
					header("Location: $redir");
				
				return $ret;
			}
		}

		return false;
	}
	
	/**
	 * Check if an object is a valid PEAR DB object
	 * @param $obj Object to check
	 */
	public static function assertIsDbObject(&$obj)
	{
		$type = get_class($obj);
		if(!preg_match('/^DB_.+$/', $type))
			throw new InvalidValueException();

		return true;
	}

	/**
	 * Sync the values to the database
	 * @param $dbConn object of PEAR DB to use for database interaction, if null $GLOBALS['dbconn'] is used
	 */
	public function sync(&$dbConn = null)
	{
		# Table name is required, otherwise generated SQL is invalid.
		if(!$this->table)
			throw new TableNotSetException();

		if($dbConn === null)
			$dbConn = $this->dbConn;

		self::assertIsDbObject($dbConn);
		$this->id = (int)$_REQUEST['id'];

		$fieldSql = array();
		foreach($this->fields as $field)
		{
			$fieldSql[] = $field->getSqlImpl();
		}

		# Implode the array, using array_diff against a null array to remove all empty elements
		$fieldSql = implode(',', array_diff($fieldSql, array('')));
		$sql = <<<EOSQL
		INSERT INTO $this->table
		SET
			id = $this->id,
			$fieldSql	
		ON DUPLICATE KEY UPDATE
			$fieldSql	
EOSQL;
		
		$res =& $dbConn->query($sql);

		if(PEAR::isError($res))
			throw new Exception($res->getMessage());
	}

	
	protected function drawHiddenFields()
	{
		parent::drawHiddenFields();
		
		echo "<input type=\"hidden\" name=\"id\" value=\"$this->id\"/>";
	}
}
?>
