<?
/**
* @ingroup BJHelpers
* @brief Linkfy is a class that does a lot of nifty text effects.
*
* It handles bbcode, sanitizing html, and a variety of effects like rainbow,
* roller, haxor, etc.
*/
class Linkify
{
	/**
	* converts certain basic html codes to equivalent bbcodes
	*
	* @param $text the html text to convert
	* @return the html text with recognized tags converted to bbcode
	*/
	public static function html2bbcode($text)
	{ 
		$text = preg_replace("/<img src='(.*)'>/i", "[img]$1[/img]", $text);
		$text = preg_replace("/<img src=\"(.*)\">/i", "[img]$1[/img]", $text);

		$text = preg_replace("/<br>/i", "\n", $text);
		$text = preg_replace("/<hr>/i", "[hr]", $text);

		$text = preg_replace("/<b>/i", "[b]", $text);
		$text = preg_replace("/<\/b>/i", "[/b]", $text);

		$text = preg_replace("/<u>/i", "[u]", $text);
		$text = preg_replace("/<\/u>/i", "[/u]", $text);

		$text = preg_replace("/<i>/i", "[i]", $text);
		$text = preg_replace("/<\/i>/i", "[/i]", $text);
		
		$text = preg_replace("/<blink>/i", "[blink]", $text);
		$text = preg_replace("/<\/blink>/i", "[/blink]", $text);

		$text = preg_replace("/(^|[^\"'=\]])(http|ftp|https):\/\/([\w.\-_:]*)(\/[\w%=+\-&?!@#$^&*~{}|,.\/]*)?/i",	"$1[url=$2://$3$4]$3[/url]", $text);
		$text = preg_replace("/(^|[^\/\]])(www\.[\w%=+\-&?!@#$^&*~{}|,.\/:]*)/i", "$1[url=http://$2]$2[/url]", $text);
		$text = preg_replace("/<a href='(.*)?'>/Usi", "[url=$1]", $text);
		$text = preg_replace("/<a href=\"(.*)?\">/Usi", "[url=$1]", $text);
		$text = preg_replace("/<\/a>/i", "[/url]", $text);

		$text = preg_replace("/<pre>(.*)?<\/pre>/Usi", "[code]$1[/code]", $text);
		$text = preg_replace("/<marquee(.*)?>(.*)?<\/marquee>/Usi", "[marquee$1]$2[/marquee]", $text);

		$text = str_replace("<ol>", "[list=1]", $text);
		$text = str_replace("</ol>", "[/list]", $text);
		$text = str_replace("<ul>", "[list]", $text);
		$text = str_replace("</ul>", "[/list]", $text);
		$text = str_replace("<li>", "[*]", $text);		
		$text = str_replace("</li>", "", $text);		

		return $text;
	} 

	/**
	* take text and format it in bbcode style.
	*
	* this takes the text, converts html to bbcode, sanitizes it, converts smilies, then converts everythign back to html
	* theoretically it will protect you from malicious text.
	*
	* @param $text the text string to format
	* @return the html formatted string suitable for display.
	*/
	public static function bbcode($text)
	{
		//trim it...
		$text = stripslashes(trim($text));
		
		//parse html2bbcode, then get rid of all html, then parse bbcode back2 html
		$text = self::html2bbcode($text);
		$text = htmlentities($text);

		//add hearts...
		$text = str_replace( "&lt;/3", "[img]lib/img/smileys/broken-heart.gif[/img]", $text );
		$text = str_replace( "&lt;3", "[img]lib/img/smileys/heart.gif[/img]", $text );
		$text = str_replace( "::bling::", "[color=green][b]\$\$\$\$[/b][/color]", $text );
		$text = str_replace( "&lt;*&gt;", "[img]lib/img/smileys/pink-star.gif[/img]", $text );
		$text = str_replace( "&lt;jesus&gt;", "[img]lib/img/smileys/jesus.gif[/img]", $text );
		$text = str_replace( "&lt;satan&gt;", "[img]lib/img/smileys/devil.gif[/img]", $text );
		$text = str_replace( "&lt;elvis&gt;", "[img]lib/img/smileys/elvis.gif[/img]", $text );
		$text = str_replace( "&lt;hoeken&gt;", "[img]lib/img/smileys/hoeken.gif[/img]", $text );
		$text = str_replace( "&lt;airplane&gt;", "[marquee][img]lib/img/smileys/airplane.jpg[/img]VROOOM![/marquee]", $text );
		$text = str_replace( "&lt;fish&gt;", "[img]lib/img/smileys/fish.gif[/img]", $text );
		$text = str_replace( "&lt;cow&gt;", "[img]lib/img/smileys/cow.jpg[/img]", $text );
		$text = str_replace( "&lt;zebra&gt;", "[img]lib/img/smileys/zebra.jpg[/img]", $text );
		$text = str_replace( "&lt;kravin&gt;", "[img]lib/img/smileys/kravin.gif[/img]", $text );
		$text = str_replace( "&lt;turtle&gt;", "[img]lib/img/smileys/turtle.gif[/img]", $text );
		$text = str_replace( "&lt;potato&gt;", "[img]lib/img/smileys/potato.gif[/img]", $text );
		$text = str_replace( "&lt;monkey&gt;", "[img]lib/img/smileys/monkey.gif[/img]", $text );
		$text = str_replace( "&lt;sun&gt;", "[img]lib/img/smileys/sun.gif[/img]", $text );
		$text = str_replace( "&lt;moon&gt;", "[img]lib/img/smileys/moon.gif[/img]", $text );
		$text = str_replace( "&lt;eclipse&gt;", "[img]lib/img/smileys/eclipse.gif[/img]", $text );
		$text = str_replace( "&lt;beer&gt;", "[img]lib/img/smileys/beer.gif[/img]", $text );
		$text = str_replace( "&lt;dino&gt;", "[img]lib/img/smileys/dino.gif[/img]", $text );
		$text = str_replace( "&lt;chicken&gt;", "[img]lib/img/smileys/chicken.gif[/img]", $text );
		$text = str_replace( "&lt;robot&gt;", "[img]lib/img/smileys/robot.gif[/img]", $text );
		$text = str_replace( "&lt;rainbowbrite&gt;", "[img]lib/img/smileys/rainbow.gif[/img]", $text );
		
		//now convert to bbcode...
		$text = self::bbcode2html($text);	
		
		//add breaks...
		$text = str_replace( "\n", "<br>\n", $text );
		
		//fix the preformatted text
		$text = preg_replace_callback("/<pre>(.*)?<\/pre>/Usi", 
			create_function(
				'$matches',
				'return Linkify::formatPre(str_replace("&quot;", "\"", $matches[1]));'
			),
			$text
		);
	
		return $text;
	}
	
	/**
	* convert bbcode to html
	*
	* @param text the bbcode formatted text you want converted to html.
	* @return the html formatted string
	*/
	public static function bbcode2html($text)
	{ 
		global $me;
		
		//get images first... dont move this
		$text = preg_replace("/\[img\](.*)?\[\/img\]/Ui", "<img src=\"$1\" border=\"0\">", $text);
		
		//next get bold and italic tags
		$text = preg_replace("/\[b\]/i", "<b>", $text);
		$text = preg_replace("/\[\/b\]/i", "</b>", $text);
		$text = preg_replace("/\[u\]/i", "<u>", $text);
		$text = preg_replace("/\[\/u\]/i", "</u>", $text);
		$text = preg_replace("/\[i\]/i", "<i>", $text);
		$text = preg_replace("/\[\/i\]/i", "</i>", $text);
		
		//do we display it??
		$text = preg_replace("/\[blink\]/i", "<blink>", $text);
		$text = preg_replace("/\[\/blink\]/i", "</blink>", $text);
		
		//now do urls and emails
		$text = preg_replace("/\[url\](.*)?\[\/url\]/Ui", '<a href="$1">$1</a>', $text);
		$text = preg_replace("/\[url=(.*)?\]/Ui", "<a href=\"$1\">", $text);
		$text = preg_replace("/\[\/url\]/", "</a>", $text);
		
		//do we display it??
		$text = preg_replace("/\[email\](.*)?(\[\/email\])/Ui", "<a href=\"mailto:$1\">$1</a>", $text);
			
		//first we get teh list entries
		$text = preg_replace("/\[\*\](.*)/", "<li>$1</li>", $text);	
		
		//strip newlines from the tags that dont want it... code, lists, etc.
		$text = preg_replace_callback("/\[list[=a1]*\](.*)?\[\/list\]/Usi",
			create_function(
				'$matches',
				'return Linkify::stripNewlines(str_replace("&quot;", "\"", $matches[0]));'
			),
			$text
		);
		
		$text = preg_replace("/\[list\](.*)?\[\/list\]/Ui", "<ul>$1</ul>", $text);
		$text = preg_replace("/\[list=a\](.*)?\[\/list\]/Ui", "<ol type=\"a\">$1</ol>", $text);
		$text = preg_replace("/\[list=1\](.*)?\[\/list\]/Ui", "<ol type=\"1\">$1</ol>", $text);

		//catch our code!!!
		$text = preg_replace("/\[code\](.*)?\[\/code\]/Usi",
			'<table class="code" cellpadding="3" cellspacing="0"><tr>' . 
			'<td><pre>$1</pre></td></tr></table>', $text);
		
		//how bout our quotes?
		if(substr_count(strtolower($text), "[quote") == substr_count(strtolower($text), "[/quote]"))
		{
			$text = str_replace('[quote]', '<table class="quote" cellpadding="3" cellspacing="0"><tr><td><i>', $text);
			$text = preg_replace('/\[quote=&quot;(.*)&quot;\]/Usi',	'<table class="quote" cellpadding="3" cellspacing="0"><tr>' . 
				"<td class=\"dk_row\"><font class=\"med-small\"><b>$1 said:</b></font></td></tr><tr><td><i>", $text);
			$text = str_replace('[/quote]', '</i></td></tr></table>', $text);
		}
		
		//get our text formatitng and shit
		$text = preg_replace("/\[color=(.*)?\]/Usi", "<span style=\"color:$1\">", $text);
		$text = preg_replace("/\[\/color\]/i", "</span>", $text);
		
		//get our text formatitng and shit
		$text = preg_replace("/\[bgcolor=(.*)?\]/Usi", "<span style=\"background-color:$1\">", $text);
		$text = preg_replace("/\[\/bgcolor\]/i", "</span>", $text);
		
		$text = preg_replace("/\[strike\]/i", "<span style=\"text-decoration: line-through;\">", $text);
		$text = preg_replace("/\[\/strike\]/i", "</span>", $text);
		
		$text = preg_replace("/\[size=(([0-2]{0,1})[0-9])?\]/Usi", "<span style=\"font-size:$1px\">", $text);
		$text = preg_replace("/\[\/size\]/i", "</span>", $text);
		
		$text = preg_replace("/\[font=(.*)?\]/Usi", "<span style=\"font-family:$1\">", $text);
		$text = preg_replace("/\[\/font\]/i", "</span>", $text);
		
		//how bout alignment??
		$text = preg_replace("/\[align=(.*)?\]/Usi", "<div style=\"text-align:$1\">", $text);
		$text = preg_replace("/\[\/align\]/i", "</div>", $text);
			
		//who doesnt like marquee?
		$text = preg_replace_callback(
			'/\[marquee(.*)?\](.*)?\[\/marquee\]/Usi',
			create_function(
				'$matches',
				'return "<marquee" . str_replace("&quot;", \'"\', $matches[1]) . ">$matches[2]</marquee>";'
			),
			$text
		);
		$text = preg_replace("/\[scroll\]/i", "<marquee>", $text);
		$text = preg_replace("/\[\/scroll\]/i", "</marquee>", $text);
			
		$text = preg_replace_callback("/\[caps\](.*)?\[\/caps\]/Usi",
			create_function(
				'$matches',
				'return Linkify::capsText(str_replace("&quot;", "\"", $matches[1]));'
			),
			$text
		);
		
		$text = preg_replace_callback("/\[haxor\](.*)?\[\/haxor\]/Usi",
			create_function(
				'$matches',
				'return Linkify::haxorText(str_replace("&quot;", "\"", $matches[1]));'
			),
			$text
		);
		
		$text = preg_replace_callback("/\[rainbow\](.*)?\[\/rainbow\]/Usi",
			create_function(
				'$matches',
				'return Linkify::rainbowText(str_replace("&quot;", "\"", $matches[1]));'
			),
			$text
		);
		
		$text = preg_replace_callback("/\[roller\](.*)?\[\/roller\]/Usi",
			create_function(
				'$matches',
				'return Linkify::rollerText(str_replace("&quot;", "\"", $matches[1]));'
			),
			$text
		);

		$text = str_replace("[hr]", "<hr>", $text);

		return $text;
	}  

	/**
	* convert text to random caps. use at your own risk ;)
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function capsText($text)
	{ 
		$fs=0;
		$direction="right";
		$rtext = "";
		$stop = false;

		//loop thru and rollerize it.
		for ($i=0; $i<strlen($text); $i++)
		{
			if($text[$i] == "<" || $text[$i] == "[")
				$stop = true;
		
			if(!$stop && $text[$i] != " ")
			{
				$l = strtolower($text[$i]);
				
				if($l >= 'a' && $l <= 'z')
				{
					//randomly convert to upper
					if(mt_rand(0,1))
						$l = strtoupper($l);
				}
				else if($l == '&')
				{
					$end = strpos($text, ";", $i);
					if(!($end === false))
					{
						$l = substr($text, $i, $end-$i+1);
						$i = $end;
					}
				}
					
				$rtext .= $l;
			}
			else
				$rtext .= $text[$i];
			
			if($text[$i] == ">" || $text[$i] == "]")
				$stop = false;
		}

		return $rtext;
	 }
	
	/**
	* convert text to roller case. use at your own risk ;)
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function rollerText($text)
	{
		$fs=0;
		$direction="right";
		$rtext = "";
		$stop = false;

		$sizes = array( 11, 14, 18, 22, 25, 28 );

		//loop thru and rollerize it.
		for ($i=0; $i<strlen($text); $i++)
		{
			if($text[$i] == "<" || $text[$i] == "[")
				$stop = true;
		
			if(!$stop && $text[$i] != " ")
			{
				$l = $text[$i];
				
				if($l == "&")
				{
					$end = strpos($text, ";", $i);
					if(!($end === false))
					{
						$l = substr($text, $i, $end-$i+1);
						$i = $end;
					}
				}
				
				$rtext .= "<span style=\"font-size:" . $sizes[$fs] . "px\">$l</span>";

				if ($fs<count($sizes)-1 && $direction=="right")
					$fs++;
				else if ($fs==count($sizes)-1)
				{ 
					$direction="left";
					$fs--;
				}
				else if ($fs==0)
				{
					$direction="right";
					$fs++;
				} 
				else if ($fs>0 && $direction=="left")
					$fs--;
			}
			else
				$rtext .= $text[$i];
			
			if($text[$i] == ">" || $text[$i] == "]")
				$stop = false;
		}

		return $rtext;
	 }
	
	/**
	* convert text to rainbow text. use at your own risk ;)
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function rainbowText($text)
	{
		$colors = array( "#ff0000", "#ff5000", "#ffa100", "#fff100", "#bbff00", "#6bff00", "#1aff00", "#00ff35", "#00ff86", "#00ffd6", "#00d6ff", "#0086ff", "#3377ff", "#4444ff", "#6b33ff", "#bb00ff", "#ff00f1", "#ff00a1", "#ff0050", "#ff0000" );	
		$fs=0;
		$direction="right";
		$rtext = "";
		$stop = false;

		//loop thru and rollerize it.
		for ($i=0; $i<strlen($text); $i++)
		{
			if($text[$i] == "<" || $text[$i] == "[")
				$stop = true;
		
			if(!$stop && $text[$i] != " ")
			{
				$l = $text[$i];
				
				if($l == "&")
				{
					$end = strpos($text, ";", $i);
					if(!($end === false))
					{
						$l = substr($text, $i, $end-$i+1);
						$i = $end;
					}
				}
				
				$rtext .= "<span style=\"color:" . $colors[$fs] . ";\">$l</span>";

				if ($fs<count($colors)-1 && $direction=="right")
					$fs++;
				else if ($fs==count($colors)-1)
				{
					$direction="left";
					$fs--;
				}
				else if ($fs==0)
				{
					$direction="right";
					$fs++;
				} 
				else if ($fs>0 && $direction=="left")
					$fs--;
			}
			else
				$rtext .= $text[$i];
			
			if($text[$i] == ">" || $text[$i] == "]")
				$stop = false;
		}

		$rtext = "<span style=\"background-color: #000000;\">$rtext</span>";
		return $rtext;
	}
	
	/**
	* convert text to h4x0r text. use at your own risk ;)
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function haxorText($text)
	{
		$text = strtolower($text);
	
		//loop thru and rollerize it.
		for ($i=0; $i<strlen($text); $i++)
		{
			if($text[$i] == "<" || $text[$i] == "[")
				$stop = true;
		
			if(!$stop)
			{
				$l = $text[$i];
				if($l == 'a')
					$r = '@';
				else if($l == 'b')
					$r = '8';
				else if($l == 'c')
					$r = '&copy;';
				else if($l == 'd')
					$r = '&ETH;';
				else if($l == "e")
					$r = '&euro;';
				else if($l == 'h')
					$r = '<b>|-|</b>';
				else if($l == 'i')
					$r = '&Icirc;';
				else if($l == 'k')
					$r = '<b>|</b>&lt;';
				else if($l == 'l')
					$r = '&pound;';
				else if($l == 'm')
					$r = '<b>|\/|</b>';
				else if($l == 'n')
					$r = '&ntilde;';
				else if($l == 'o')
					$r = '&Oslash;';
				else if($l == 'p')
					$r = '&para;';
				else if($l == 'r')
					$r = '&reg;';
				else if($l == 's')
					$r = '$';
				else if($l == 't')
					$r = '<b>&dagger;</b>';
				else if($l == 'u')
					$r = '&uuml;';
				else if($l == 'v')
					$r = '<b>\/</b>';
				else if($l == 'w')
					$r = '<b>\/\/</b>';
				else if($l == 'x')
					$r = '&times;';
				else if($l == 'y')
					$r = '&yen;';
				else if($l == '?')
					$r = '&iquest;';
				else
					$r = $l;
			
				$rtext .= $r;
			}
			else
				$rtext .= $text[$i];
			
			if($text[$i] == ">" || $text[$i] == "]")
				$stop = false;
		}

		return $rtext;
	}
	
	/**
	* strip newlines from teh text.
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function stripNewlines($text)
	{
		return str_replace("\n", "", $text);
	}

	/**
	* format the text for use in a <pre>
	*
	* @param $text the text to convert
	* @return the converted text.
	*/
	public static function formatPre($text)
	{
		return "<pre>" . str_replace("<br>", "", $text) . "</pre>";
	}
	
	/**
	 * Draws an email in a spam-bot/web-spider safe fashion.  Protects your email.
	 *
	 * @param $email the email to print.
	 * @param $display the text to display
	 * @return the formatted email
	 */
	public static function email($email, $display = null)
	{
		$emailSplit = explode("@", $email);

		if (count($emailSplit) == 2)
		{
			$result = "<script type=\"text/javascript\"><!--\n";
			$result .= "document.write('<a href=\"mailto:' + (['$emailSplit[0]', '$emailSplit[1]'].join('@')) + '\">' + ";

			if ($display)
				$result .= "'" . str_replace("\n", "\\n", str_replace("'", "\\'", $display)) . "'";
			else
				$result .= "(['$emailSplit[0]', '$emailSplit[1]'].join('@'))";

			$result .= " + '<\/a>');";
			$result .= "//--></script>";

			$textEmail = str_replace(".", " (dot) ", $email);
			$textEmail = str_replace("@", " (at) ", $textEmail);

			$result .= "<ins><noscript><div>";
			if ($display)
				$result .= "$display ($textEmail)";
			else
				$result .= $textEmail;
			$result .= "</div></noscript></ins>";

			return $result;
		}

		return '';
	}
}
?>
