<?
require_once("phpmailer/class.phpmailer.php");

/**
* @ingroup BJHelpers
* @brief Simple, unified mail interface.
*
* Mail is a singleton class that handles sending mail in an easy, __autoload friendly fashion. 
*/
class Mail
{
	/**
	* this is our mailer class instance.
	*/
	private static $mailer;
	
	/**
	* no instantiation... singleton class
	*/
	private function __construct()
	{
	}
	
	/**
	* send our mail to an address
	*
	* @param $to the to email address, or a comma separated list of them.
	* @param $subject the subject of the email
	* @param $message the body of the email
	* @param $html the html body of the email
	* @param $from the from email address
	*
	* @return the result of the email sending.
	*/
	public static function send($to, $subject, $message, $html = null, $from = null)
	{
		return self::$mailer->mail($to, $subject, $message, $html, $from);
	}

	/**
	* set our mailer object.
	*
	* @param $mailer a phpmailer derived object
	*/
	public static function setMailer(phpmailer $mailer)
	{
		self::$mailer = $mailer;
	}
	
	/**
	* get our mailer object.
	*
	* @return our phpmailer derived object
	*/
	public static function getMailer()
	{
		return self::$mailer;
	}
}

/**
* this is our wrapper class for phpmailer. 
*/
class MyMail extends phpmailer 
{ 
    // Set default variables for all new objects

	function __construct()
	{
		$this->Sender   = Config::get('mailer_from');
		$this->From     = Config::get('mailer_from');
		$this->FromName = Config::get('mailer_from_sender');
		$this->Hostname = Config::get('mailer_from_host');
		$this->WordWrap = 100;
		$this->isMail();
	}
 
	function mail($to, $subject, $message, $html = null, $from = null)
	{
		if ($from !== null)
		{
			$this->From = $from;
			$this->Sender = $from;
		}

		if ($html === null)
			$html = nl2br($message);

		$this->ClearAddresses();
		$this->ClearAttachments();
		
		$this->Body = $html;
		$this->Subject = $subject;
		$this->AltBody = $message;

		if (is_object($to))
			$this->AddAddress($to->email, $to->getName());
		else
			$this->AddAddress($to);

		if(!$this->Send())
		        echo "There has been a mail error sending to $to <br/>";
	}
 
	// Replace the default error_handler
    function error_handler($msg)
	{
        throw new Exception("<b>Mail Error: $msg</b>"); 
    }
}

//create our global mailer...
Mail::setMailer(new MyMail());
?>
