<?
/**
* @ingroup BaseObjects
* @brief Used with the BaseUser class to give an easy, quick way to alert the user to a variety of things.
*
* You add alerts, which consist of a message and a link.  They can then view
* the alert and it takes the to them appropriate page.  They are automatically
* deleted after viewed.
*/
class BaseAlert extends MyObject
{
	/**
	* @see BaseObject::__construct
	*/
	public function __construct($data = null, $table = 'alerts')
	{
		parent::__construct($data, $table);
	}

	/**
	* can i view the alert?
	*
	* @return boolean if we can/cant view it.
	*/
	public function canView()
	{
		global $me;
		
		return ($this->user_id == $me->id);
	} 
	
	/**
	* can i delete the alert?
	*
	* @return boolean if we can/cant delete it.
	*/
	public function canDelete()
	{
		return $this->canView();
	}
	
	/**
	* get the edit page xml
	*
	* in this special case, we return nothing since there is no edit page.
	*/
	protected function getEditPageXml()
	{
		return '';
	}

	/**
	* draw our delete page, no confirmation
	*/
	public function drawDeletePage()
	{
		$this->delete();

		echo 'Alert deleted.';
	}

	/**
	* prep the view page.
	*
	* actually with this one, it handles all the work, since we're redirecting
	* them afterwards. it loads the alert, gets the url, deletes, then
	* redirects.
	*/
	public function initViewPage()
	{
		//ya gotta log in buddy
		$this->assertLogin();
		
		//do our checks.
		parent::initViewPage();

		//this alert will self desctruct
		$this->delete();

		//send them home.
		Util::redirect($this->url);
	}

	/**
	* gets and/or links our alert text
	*
	* @see BaseObject::getName
	*/
	public function getName($link = false)
	{
		return parent::getName($link, $this->text);
	} 

	/**
	* draws our alert line
	*/
	public function drawLine()
	{
		$this->getName(true);
	}

	/**
	* draws our alert as a row... calls drawLine()
	*/
	public function drawRow()
	{
		$this->drawLine();
	} 

	/**
	* gets our alert as an rss item
	*
	* @see BaseObject::getRssItem()
	*/
	public function getRssItem()
	{
		$item = parent::getRssItem();

		$item->title = $this->getName();
		$item->description = $this->getName(true);

		return $item;
	}
	
	/**
	* gets our alert for data export
	*
	* @see BaseObject::getPublicData()
	*/
	public function getPublicData()
	{
		$data = $this->getPublicData();

		$data['alert'] = $this->getName(true);
		$data['text'] = $this->text;
		$data['url'] = $this->url;

		return $data;
	}
	
	/**
	* get an array of fields in the table that stores the alerts
	*
	* @return an array of field definitions with keys that are the same as the field name.
	*/
	public function getCreateFieldsArray()
	{
		$arr = parent::getCreateFieldsArray();

		unset($arr['add_date']);
		unset($arr['edit_date']);
		
		$arr['url'] = "url varchar(255) default '' not null";
		$arr['text'] = "`text` varchar(255) default '' not null";

		return $arr;
	}

	/**
	* get an array of indexes on fields in the alert table
	*
	* @return a keyed array with keys as the field/index name and values as the index creation sql.
	*/
	public function getCreateIndexesArray()
	{
		$arr = parent::getCreateIndexesArray();

		unset($arr['add_date']);
		unset($arr['edit_date']);
	
		return $arr;
	}
}
?>
