<?
/**
* @ingroup ObjectManagers
* @brief Manage events with calendar functionality
*
* A calendar based manager for dealing with events.  Has calendar view, day
* view, upcoming, past, etc.  Very handy.
*
* BaseCalendar uses a heavily modified version of the Calendar class from
* http://www.cascade.org.uk/software/php/calendar/index.php
*/
class BaseCalendar extends MyManager
{
	/**
	* The start day of the week. This is the day that appears in the first
	* column of the calendar. Sunday = 0.
	*/
	public $startDay = 0;

	/**
	* The start month of the year. This is the month that appears in the first
	* slot of the calendar in the year view. January = 1.
	*/
	public $startMonth = 1;

	/**
	* The labels to display for the days of the week. The first entry in this
	* array represents Sunday.
	*/
	public $dayNames = array("S", "M", "T", "W", "T", "F", "S");
	
	/**
	* The labels to display for the months of the year. The first entry in this
	* array represents January.
	*/
	public $monthNames = array("January", "February", "March", "April", "May", "June",
							"July", "August", "September", "October", "November", "December");
							
	/**
	* The number of days in each month. You're unlikely to want to change
	* this... The first entry in this array represents January.
	*/
	public $daysInMonth = array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

	/**
	* what day it is
	*
	* @todo wtf is this for?
	*/
	public $day = 0;
	
	/**
	* this is where we store the day => count of events in that day
	*/
	public $dayCount = array();

	/**
	* what size of calendar are we showing?  large or small
	*/
	public $size;

	/**
	* our array of BaseEvent objects
	*/
	public $events = null;

	/**
	* setup our calendar object
	*
	* @see ObjectManager::__construct()
	*/
	public function __construct($class = 'Event')
	{
		parent::__construct($class);
		
		if (!$this->page)
			$this->page = 'calendar';
	}
	
	/**
	* add our calendar pages in
	*
	* upcoming - events in the future
	* past - events in the past
	* popular - popular events
	* calendar - show calendar / month view
	* date - show events on a date
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();
		$xml .= "<page name=\"upcoming\"/>\n";
		$xml .= "<page name=\"past\"/>\n";
		$xml .= "<page name=\"popular\"/>\n";
		$xml .= "<page name=\"calendar\">\n";
		$xml .= "\t<param name=\"month\" regex=\"/^[0-9]{1,2}$/\"/>\n";
		$xml .= "\t<param name=\"year\" regex=\"/^[0-9]{4}$/\"/>\n";
		$xml .= "</page>\n";
		$xml .= "<page name=\"date\">\n";
		$xml .= "\t<param name=\"year\" regex=\"/^[0-9]{4}$/\"/>\n";
		$xml .= "\t<param name=\"month\" regex=\"/^[0-9]{1,2}$/\"/>\n";
		$xml .= "\t<param name=\"day\" regex=\"/^[0-9]{1,2}$/\"/>\n";
		$xml .= "</page>\n";
		return $xml;
	}

	/**
	* get params for search page
	*
	* montth, day, year, event start, and stop are all added
	*/
	public function getSearchParamXml()
	{
		$xml = parent::getSearchParamXml();
		$xml .= <<<XML
			<param name="month" type="int"/>
			<param name="day" type="int"/>
			<param name="year" type="int"/>
			<param name="event_date_start" desc="start date for your query, eg 10-22-1983. it will get all the events happening on or after it" regex="/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})?$/"/>
			<param name="event_date_stop" desc="end date for your query. it will get all the events happening on or before it.  used with start date you can specify a range."  regex="/^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})?$/"/>
XML;

		return $xml;
	}

	/**
	* special select SQL
	* @see ObjectManager::getSearchSelect()
	*/
	public function getSearchSelect($params)
	{ 
		$sql = parent::getSearchSelect($params);

		$sql .= ", count(eg.user_id) AS guests ";

		return $sql;
	}

	/**
	* special from SQL, join on guest table
	*
	* @see ObjectManager::getSearchFrom()
	*/
	public function getSearchFrom($params)
	{
		$from = parent::getSearchFrom($params);

		$from .= " LEFT OUTER JOIN {$this->object->guestTable} eg ON o.id = eg.event_id ";

		return $from;
	}
	
	/**
	* special where SQL: dates, etc.
	*
	* @see ObjectManager::getSearchWhere()
	*/
	public function getSearchWhere($params)
	{
		global $me;

		$where = parent::getSearchWhere($params);

		//get our params..
		$year = $params['year'];
		$month = str_pad($params['month'], 2, "0", STR_PAD_LEFT);
		$day = str_pad($params['day'], 2, "0", STR_PAD_LEFT);

		//check and make sql.
		if ($year && (int)$month && (int)$day)
			$where .= " AND o.start_date <= '$year-$month-$day' AND o.end_date >= '$year-$month-$day'";
		else if ($year && (int)$month)
			$where .= " AND (
				(o.start_date >= '$year-$month-01' && o.start_date <= '$year-$month-31') OR
				(o.end_date >= '$year-$month-01' && o.end_date <= '$year-$month-31')
			)";
		else if ($year)
			$where .= " AND (
				(o.start_date >= '$year-01-01' && o.start_date <= '$year-12-31') OR
				(o.end_date >= '$year-01-01' && o.end_date <= '$year-12-31')
			)";

		//when is our event start date??
		$start = $params['event_date_start'];
		if ($start)
			$where .= " AND (o.start_date >= '$start' OR o.end_date >= '$start')";
		
		//what about event stop date?
		$stop = $params['event_date_stop'];
		if ($stop)
			$where .= " AND o.start_date <= '$stop'";

		//what about invite only stuff?
		if ($this->object->hasField('invite_only'))
			$where .= " AND (o.invite_only = 0 OR (o.invite_only = 1 AND eg.user_id = '$me->id')) ";
		
		return $where;
	}

	/**
	* special order SQL
	* @see ObjectManager::getSearchOrder()
	*/
	public function getSearchOrder($params)
	{
		//get our sort...
		$sort = $params['sort'];
		$order = $params['order'];

		//if we dont have order and sort... add date desc!
		if (!$sort)
			$params['sort'] = 'start_date';
		if (!$order)
			$params['order'] = 'ASC';

		return parent::getSearchOrder($params);
	}
	
	/**
	* get fields to sort by
	* @see ObjectManager::getSortFieldsArray()
	*/
	protected function getSortFieldsArray()
	{
		$sort = parent::getSortFieldsArray();

		$sort['name'] = 'sort by number of name';
		$sort['guests'] = 'sort by number of guests';
		$sort['start_date'] = 'sort by number of start date';
		$sort['end_date'] = 'sort by number of end date';

		return $sort;
	}

	/**
	* group by our event id
	*/
	public function getSearchGroupBy($params)
	{
		return ' GROUP BY o.id ';
	}

	/**
	* do our search.  @see ObjectManager::search()
	*/
	public function search($params, $limit = null)
	{
		if (!$params['month'] && !$params['year'] && !$params['event_date_start'] && !$params['event_date_stop'])
			$params['event_date_start'] = date('Y-m-d');
		return parent::search($params, $limit);
	}

	/**
	* setup a default month / date thingie based on params
	*/
	public function findDate()
	{
		if ($this->params('month'))
			$month = (int)$this->params('month');
		if ($this->params('year'))
			$year = (int)$this->params('year');
		if ($this->params('day'))
			$day = (int)$this->params('day');
	
		if (!$month)
			$month = date('m');
		if (!$year)
			$year = date('Y');

		$this->setParam('month', $month);
		$this->setParam('year', $year);
	}

	/**
	* prep main page, redirect to calendar
	*/
	public function initMainPage()
	{
		Util::redirect(".calendar");
	}

	/**
	* prep search page
	*/
	public function initSearchPage()
	{
		parent::initSearchPage();
		
		$this->pageTitle = 'Calendar of Events';
	} 

	/**
	* prep our calendar page
	*
	* find date, title, etc.
	*/
	public function initCalendarPage()
	{
		//what date?
		$this->findDate();
		
		//box and title...
		$this->pageTitle = 'Calendar of Events - ' . $this->monthNames[(int)$this->params('month')-1];
	}

	/**
	* prep our date page
	*/
	public function initDatePage()
	{
		global $me;

		$this->pageTitle = "Events for " . $me->formatDate($this->params('year') . "-" . $this->params('month') . "-" . $this->params('day'));
	}

	/**
	* prep our upcoming page
	*/
	public function initUpcomingPage()
	{
		$this->setParam('event_date_start', date('Y-m-d'));

		$this->initSearchPage();
		
		$this->pageTitle = 'Upcoming Events';
	}

	/**
	* prep our popular page
	*/
	public function initPopularPage()
	{
		$this->setParam('event_date_start', date('Y-m-d'));
		$this->setParam('sort', 'guests');
		$this->setParam('order', 'desc');

		$this->initSearchPage();
		
		$this->pageTitle = 'Popular Events';
	}
	
	/**
	* prep our past page
	*/
	public function initPastPage()
	{
		$this->setParam('event_date_stop', date('Y-m-d'));
		$this->setParam('order', 'desc');
		$this->initSearchPage();
		
		$this->pageTitle = 'Past Events';
	}

	/**
	* draw our date page
	*/
	public function drawDatePage()
	{
		$year = $this->params('year');
		$month = $this->params('month');
		$day = $this->params('day');
	
		echo "<p>" . $this->object->getLink(".edit?date=$year-$month-$day", "Post a new event on this day.");
		echo " | " . $this->getLink(".calendar?month=$month&year=$year", "Back to " . date("M Y", strtotime("$year-$month-$day")));
		echo "</p>";
		
		$rs = $this->search($this->params());
		if (count($rs))
			$this->drawResults($rs);
		else
			echo "<p>Sorry, there aren't any events on this day yet.</p>";

	}
	
	/**
	* draw our calendar page
	*/
	public function drawCalendarPage()
	{
		$this->drawSearchForm();

		echo "<div id=\"bigCalendar\">";
		$this->drawCalendar($this->params('month'), $this->params('year'));
		echo "</div>";

		//draw our events
		$events = $this->getMonthEvents($month, $year);
		$this->drawResults($events);
	} 

	/**
	* draw our upcoming events page
	*/
	public function drawUpcomingPage()
	{
		$this->drawSearchPage();
	}
	
	/**
	* draw our popular events page
	*/
	public function drawPopularPage()
	{
		$this->drawSearchPage();
	}
	
	/**
	* draw our past events page
	*/
	public function drawPastPage()
	{
		$this->drawSearchPage();
	}

	/**
	* draw our calendar table html
	*
	* @param $month the month to draw (1-12)
	* @param $year the year to draw (4 digit year)
	* @param $size the size of the calendar.  big or small
	*/
	public function drawCalendar($month, $year, $size = 'big')
	{
		//remember our size.
		$this->size = $size;
		
		//get our month events...
		$this->setMonthEvents($month, $year);

		//draw our calendar.
		echo $this->getMonthView($month, $year);
	}

	/**
	* get all the events for this month
	*
	* @param $month the month to look up (1-12)
	* @param $year the year to look up (4 digit year)
	* @return an array of BaseEvent objects
	*/
	protected function getMonthEvents($month, $year)
	{
		if ($this->events === null)
		{
			//check to see what events are on this calendar month.
			return $this->search(array(
				'month' => $month,
				'year' => $year,
			));
	 	}
		else
			return $this->events;
	} 
	
	/**
	* setup our month events and per day counts
	*
	* @param $month month of year (1-12)
	* @param $year the year to lookup (4 digit number)
	*/
	protected function setMonthEvents($month, $year)
	{ 
		$this->events = $this->getMonthEvents($month, $year);
		
		//did we get any?
		if (count($this->events))
		{
			//loop thru and put them in our array.
			foreach ($this->events AS $ob)
			{
				if (preg_match('/(\d{4})-(\d{2})-(\d{2})/', $ob->start_date, $m))
					$this->dayCount[(int)$m[3]]++;			
	 		}
	 	}
	}

	/**
	* Get the array of strings used to label the days of the week. This array
	* contains seven elements, one for each day of the week. The first entry in
	* this array represents Sunday.
	*
	* @return an array of day names
	*/
	function getDayNames()
	{
		return $this->dayNames;
	}
	

	/**
	* Set the array of strings used to label the days of the week. This array
	* must contain seven elements, one for each day of the week. The first
	* entry in this array represents Sunday. 
	*
	* @param $names an array of day names
	*/
	function setDayNames($names)
	{
		$this->dayNames = $names;
	}
	
	/**
	* Get the array of strings used to label the months of the year. This array
	* contains twelve elements, one for each month of the year. The first entry
	* in this array represents January.
	*
	* @return an array of month names
	*/
	function getMonthNames()
	{
		return $this->monthNames;
	}
	
	/**
	* Set the array of strings used to label the months of the year. This array
	* must contain twelve elements, one for each month of the year. The first
	* entry in this array represents January.
	*
	* @param $names an array of month names
	*/
	function setMonthNames($names)
	{
		$this->monthNames = $names;
	}
	
	/**
	* Gets the start day of the week. This is the day that appears in the first
	* column of the calendar. Sunday = 0.
	*
	* @return the start day
	*/
	function getStartDay()
	{
		return $this->startDay;
	}
	
	/**
	* Sets the start day of the week. This is the day that appears in the first
	* column of the calendar. Sunday = 0.
	*
	* @param $day the start day
	*/
	function setStartDay($day)
	{
		$this->startDay = $day;
	}
	
	
	/**
	* Gets the start month of the year. This is the month that appears first in
	* the year view. January = 1.
	*
	* @return the start month
	*/
	function getStartMonth()
	{
		return $this->startMonth;
	}
	
	/**
	* Sets the start month of the year. This is the month that appears first in
	* the year view. January = 1.
	*
	* @param $month the start month
	*/
	function setStartMonth($month)
	{
		$this->startMonth = $month;
	}
	
	/**
	* Return the URL to link to in order to display a calendar for a given
	* month/year. You must override this method if you want to activate the
	* "forward" and "back" feature of the calendar.
	*
	* Note: If you return an empty string from this function, no navigation
	* link will be displayed. This is the default behaviour.
	*
	* If the calendar is being displayed in "year" view, $month will be set to
	* zero.
	*
	* @param $month the month to link to
	* @param $year the year to link to
	* @param the <a> html tag link
	*/
	function getCalendarLink($month, $year)
	{
		return $this->getUrl(".calendar?month=$month&year=$year");
	} 
	
	/**
	* Return the URL to link to  for a given date. You must override this
	* method if you want to activate the date linking feature of the calendar.
	*
	* Note: If you return an empty string from this function, no navigation
	* link will be displayed. This is the default behaviour.
	*
	* @param $day the day to link to
	* @param $month the month to link to
	* @param $year the year to link to
	* @param the <a> html link tag
	*/
	function getDateLink($day, $month, $year)
	{
		return $this->getUrl(".date?year=$year&month=$month&day=$day");
	}

	/**
	* get content to put in the calendar day box for that day.
	*
	* defaults to the number of events on that day.
	*
	* @param $day the day of content
	* @param $month the month to check
	* @param $year the year to check
	* @return the content you want to display on that day.
	*/
	function getDateContent($day, $month, $year)
	{
		$s = $day;
	
		//how many events?
		if ($this->dayHasEvent($day) && $this->size == 'big')
			$s .= "<br/>" . Util::pluralize($this->object->english, $this->dayCount[$day], true);
	
		return $s;
	}
	
	/**
	* Return the calendar HTML for a specified month
	*
	* @param $month the month to display
	* @param $year the year to display
	* @param $showYear do we show the year or not
	*
	* @return the markup for that calendar month
	*/
	function getMonthView($month, $year, $showYear = true)
	{
		//start our table.
		$s = "<table class=\"calendar\">\n";
		
		//adjust our date so its correct.
		$a = $this->adjustDate($month, $year);
		$month = $a[0];
		$year = $a[1];        

		//get our header.
		$s .= $this->getMonthHeader($month, $year, $showYear);
		$s .= $this->getMonthBody($month, $year);
		
		$s .= "</table>\n";
		
		return $s;  	
	}
	
	/**
	* Format the month header row.
	*
	* @param $month the month to draw
	* @param $year the year to draw
	* @param $showYear show the year or not
	*
	* @return an html formatted string.
	*/
	function getMonthHeader($month, $year, $showYear = true)
	{
		//our monthname
		$monthName = $this->monthNames[$month - 1];
		
		//get our previous/next months.
		$prev = $this->adjustDate($month - 1, $year);
		$next = $this->adjustDate($month + 1, $year);
		
		//do we want links to prev/next?
		if ($showYear)
		{
			$prevMonth = $this->getCalendarLink($prev[0], $prev[1]);
			$nextMonth = $this->getCalendarLink($next[0], $next[1]);
		}
		else
		{
			$prevMonth = "";
			$nextMonth = "";
		}
		
		//set our header.
		$header = $monthName . (($showYear) ? " " . $year : "");
		
		$s  = "<tr>\n";
		$s .= "<th valign=\"top\">" . (($prevMonth == "") ? "&nbsp;" : "<a href=\"$prevMonth\">&lt;&lt;</a>")  . "</th>\n";
		$s .= "<th valign=\"top\" colspan=\"5\">$header</th>\n"; 
		$s .= "<th valign=\"top\">" . (($nextMonth == "") ? "&nbsp;" : "<a href=\"$nextMonth\">&gt;&gt;</a>")  . "</th>\n";
		$s .= "</tr>\n";
		
		$s .= "<tr>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+1)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+2)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+3)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+4)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+5)%7] . "</th>\n";
		$s .= "<th>" . $this->dayNames[($this->startDay+6)%7] . "</th>\n";
		$s .= "</tr>\n";

		return $s;
	}
		
	/**
	* get the html markup for a month
	*
	* @param $month the month to get
	* @param $year the year to get
	*/
	protected function getMonthBody($month, $year)
	{
		//init return string.
		$s = '';
		
		//find out our day count and general date info.
		$daysInMonth = $this->getDaysInMonth($month, $year);
		$date = getdate(mktime(12, 0, 0, $month, 1, $year));
		
		//get our first date
		$first = $date["wday"];
		
		// We need to work out what date to start at so that the first appears in the correct column
		$d = $this->startDay + 1 - $first;
		while ($d > 1)
			$d -= 7;

		// Make sure we know when today is, so that we can use a different CSS style
		$today = getdate(time());
		
		//loop until we run out of days.
		while ($d <= $daysInMonth)
		{
			$s .= "<tr>\n";       
			
			//each week day.
			for ($i = 0; $i < 7; $i++)
			{
				//if its today, we want a special class.
				if ($this->dayHasEvent($d))
					$s .= '<td class="hasEvent">';
				else if ($year == $today["year"] && $month == $today["mon"] && $d == $today["mday"])
					$s .= '<td class="today">';
				else
					$s .= '<td>';

				//if its in the month... draw the day.	
				if ($d > 0 && $d <= $daysInMonth)
				{
					$link = $this->getDateLink($d, $month, $year);
					$content = $this->getDateContent($d, $month, $year);
					if ($link)
						$s .= "<a href=\"$link\">$content</a>";
					else
						$s .= $content;
				}
				//otherwise leave it blank.
				else
				{
					$s .= "&nbsp;";
				}

				//end it
				$s .= "</td>\n";

				//next day.
				$d++;
			}

			//next row
			$s .= "</tr>\n";    
		}

		return $s;
	}

	/**
	* do we have an event on this day on our calendar?
	*
	* @param $d the day to check (1-31)
	* @return boolean result
	*/
	public function dayHasEvent($d)
	{
		return ($this->dayCount[$d]);
	}
	
	/**
	* Return the HTML for a specified year
	*
	* @param $year the year to draw
	* @return html for displaying n entire year of calendar
	*/
	function getYearView($year)
	{
		$s = "";
		$prev = $this->getCalendarLink(0, $year - 1);
		$next = $this->getCalendarLink(0, $year + 1);
		
		$s .= "<table class=\"calendar\" border=\"0\">\n";
		$s .= "<tr>";
		$s .= "<td align=\"center\" valign=\"top\" align=\"left\">" . (($prev == "") ? "&nbsp;" : "<a href=\"$prev\">&lt;&lt;</a>")  . "</td>\n";
		$s .= "<td class=\"calendarHeader\" valign=\"top\" align=\"center\">" . (($this->startMonth > 1) ? $year . " - " . ($year + 1) : $year) ."</td>\n";
		$s .= "<td align=\"center\" valign=\"top\" align=\"right\">" . (($next == "") ? "&nbsp;" : "<a href=\"$next\">&gt;&gt;</a>")  . "</td>\n";
		$s .= "</tr>\n";
		$s .= "<tr>";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(0 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(1 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(2 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "</tr>\n";
		$s .= "<tr>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(3 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(4 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(5 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "</tr>\n";
		$s .= "<tr>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(6 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(7 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(8 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "</tr>\n";
		$s .= "<tr>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(9 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(10 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "<td class=\"calendar\" valign=\"top\">" . $this->getMonthHTML(11 + $this->startMonth, $year, 0) ."</td>\n";
		$s .= "</tr>\n";
		$s .= "</table>\n";
		
		return $s;
	}
	
	/********************************************************************************
		The rest are private methods. No user-servicable parts inside.
		You shouldn't need to call any of these functions directly.
	*********************************************************************************/


	/**
	* Calculate the number of days in a month, taking into account leap years.
	*
	* @param $month the month to count
	* @param $year the year to count
	* @return the number of days in a month
	*/
	protected function getDaysInMonth($month, $year)
	{
		if ($month < 1 || $month > 12)
		{
			return 0;
		}
   
		$d = $this->daysInMonth[$month - 1];
   
		if ($month == 2)
		{
			// Check for leap year
			// Forget the 4000 rule, I doubt I'll be around then...
		
			if ($year%4 == 0)
			{
				if ($year%100 == 0)
				{
					if ($year%400 == 0)
					{
						$d = 29;
					}
				}
				else
				{
					$d = 29;
				}
			}
		}
	
		return $d;
	}

	/**
	* Generate the HTML for a given year
	* 
	* @todo remove this or make it work
	* @param $year the year to get
	* @return the html code to display the year
	*/
	protected function getYearHTML($year)
	{
	}

	/**
	* Adjust dates to allow months > 12 and < 0. Just adjust the years
	* appropriately. e.g. Month 14 of the year 2001 is actually month 2 of year
	* 2002.
	*
	* @param $month the month to adjust
	* @param $year the year to adjust
	* @return an array with index 0 as the proper month, and index 1 as the proper year
	*/
	protected function adjustDate($month, $year)
	{
		$a = array();  
		$a[0] = $month;
		$a[1] = $year;
		
		while ($a[0] > 12)
		{
			$a[0] -= 12;
			$a[1]++;
		}
		
		while ($a[0] <= 0)
		{
			$a[0] += 12;
			$a[1]--;
		}
		
		return $a;
	}
}
?>
