<?
/**
* @ingroup BaseObjects
* @brief BaseComment is an object used for comment threads.
*
* A class that simplifies adding, editing, deleting, replying, and displaying
* of comments.  It makes adding comments to content very easy.
*/
abstract class BaseComment extends MyObject
{
	public $class = '';

	/**
	* The constructor...
	*
	* @param $data see BaseObject constructor
	* @param $class the name of the class the comment is on... dont put
	* 'comment' in here.... unless its a comment comment.
	*/
	function __construct($data, $class)
	{
		parent::__construct($data, "comments_" . strtolower($class));

		$this->class = $class;
		$this->english = "comment";

		if (!class_exists($this->class))
			throw new Exception("Class $this->class doesn't exist!");

		//add in our fulltext fields
		$this->fullTextFields[] = 'body';
	}

	/**
	* Initialize teh delete page.
	*/
	function initDeletePage()
	{
		parent::initDeletePage();

		$this->pageTitle = 'Delete Comment';
	}

	/**
	* Redirect to our original object the comment was posted on.
	*/
	function deletePost()
	{
		parent::deletePost($this->getUrl("$this->class.view?id=$this->content_id"), true);
	}

	/**
	* do the actual deletion.  also save the abandoned child comments.
	*/
	function delete()
	{
		dbExecute("
			UPDATE $this->tableName
			SET parent_id = '$this->parent_id'
			WHERE parent_id = '$this->id'
		");
	
		parent::delete();
	}

	/**
	* initialize the edit page.
	*/
	function initEditPage()
	{
		parent::initEditPage();

		if ($this->id)
			$this->pageTitle = 'Edit Comment';
		else
			$this->pageTitle = 'Add New Comment';

		//if its new... get our ids.
		if (!$this->id)
		{
			$parentId = $this->params('parent_id');
			$contentId = $this->params('content_id');
			
			if ($parentId)
			{
				$class = get_class($this);
				$parent = new $class($parentId);
				
				$this->parent_id = $parentId;
				$this->content_id = $parent->content_id;
			}
			else if ($contentId)
			{
				$this->parent_id = 0;
				$this->content_id = $contentId;
			}
			else
				throw new PageError('You must supply either a parent id or a content id.');
		}
	}

	/**
	* Add our comment fields... pretty basic.  body and submit button.
	*/
	function editFormAddFields($form)
	{
		$form->add('TextAreaField', 'body', array(
			"required" => true,
			"title" => 'Comment',
			"class" => 'BaseComment'
		));
		if ($this->id)
			$form->addSubmit('Save Comment');
		else
			$form->addSubmit('Post Comment');
	}

	/**
	* creates a reply form for inline displaying with our nifty javascript.
	*
	* @return the reply form as a Form object.
	*/
	function createReplyForm($contentId = null)
	{
		$form = new Form();

		if ($this->id)
		{
			$form->action = $this->getUrl(".edit?parent_id=$this->id");
			$form->name = "replyForm_$this->id";
			$form->id = "replyForm_$this->id";
		}
		else
			$form->action = $this->getUrl(".edit?content_id=$contentId");

		$form->add('TextAreaField', 'body', array(
			'title' => '',
			'required' => true,
			'class' => 'BaseComment',
			'height' => '75px',
			'width' => '95%'
		));

		if ($this->id)
			$form->addSubmit('Post Reply');
		else
			$form->addSubmit('Post New Comment');

		return $form;
	}

	/**
	* get the edit page parameters xml
	*
	* @return an xml string of edit page xml
	*/
	function getEditPageParamXml()
	{
		$xml =  <<<XML
	<param name="subject" desc="title of your comment.  pretty important." />
	<param name="body" desc="comment main body" />
	<param name="content_id" desc="id of content in the module you're commenting on" />
	<param name="parent_id" desc="id of the parent comment" />
XML;
		return $xml;
	}

	/**
	* emails the author of original content and also poster you replied to.  then redirects to main content.
	*/
	function editPagePostSuccess($form)
	{
		$this->emailNotification();
		$this->emailReply();
		
		Util::redirect($this->getUrl("$this->class.view?id=$this->content_id"));
	}

	/**
	* emails the author of the original content notifying them of a new comment
	* on their stuff.  If you want this enabled, the user table must have
	* 'email_comments' as a tinyint(1)
	*/
	function emailNotification()
	{
		//get our people and our stuff
		$content = new $this->class($this->content_id);

		//we dont want to send to ourself.
		if ($this->creator->id != $content->creator->id)
		{
			//does the this->creator want comment notices?
			if ($content->creator->email_comments)
			{
				$subject = "New $this->class comment";

				$body  = $this->creator->getName() . " has posted a new comment on your $this->class.\n\n";
				$body .= "You can view it here:\n\n";
				$body .= $content->creator->getTicket(array(
					'module' => $this->class,
					'page' => 'view',
					'id' => $content->id
				)) . "\n\n";
				$body .= "Thank you,\nThe Happy MyKin Comment Robot\n\n";
				$body .= "PS.  You can turn these emails off here:\n\n";
				$body .= $content->creator->getUrl(array('page' => 'emailprefs'), null, true);
				
				$html  = $this->creator->getName() . " has posted a new comment on your $this->class.<br/><br/>";
				$html .= $content->creator->getTicket(array(
					'module' => $this->class,
					'page' => 'view',
					'id' => $content->id
				), 'You can view it here') . ".<br/><br/>";
				$html .= "Thank you,<br/>The Happy MyKin Comment Robot<br/><br/>";
				$html .= "PS.  You can turn these emails off " . $content->creator->getLink('.emailprefs', 'here', null, true);
				
				$content->creator->mail($subject, $body, $html);
			}
		}
	}

	/**
	* if the comment was in response to another comment, will email author of
	* the comment.  that is if the user table has a field called
	* "email_comment_replies" of type tinyint(1)
	*/
	function emailReply()
	{
		//get our stuff
		$content = new $this->class($this->content_id);
		$comClass = get_class($this);
		$parent = new $comClass($this->parent_id);
		
		//if the parent author is the same as content auth... they'll get an email on the content comment.
		//also, dont send an email if they are the this->creator.
		if ($parent->creator->id != $content->creator->id && $parent->creator->id != $this->creator->id)
		{
			//does the this->creator want comment notices?
			if ($parent->creator->email_comment_replies)
			{
				$subject = "New comment reply";

				$body  = $this->creator->getName() . " has posted a new reply to one of your $this->class comments.\n\n";
				$body .= "You can view it here:\n\n";
				$body .= $parent->creator->getTicket(array(
					'module' => $this->class,
					'page' => 'view',
					'id' => $content->id
				)) . "\n\n";
				$body .= "Thank you,\nThe Happy MyKin Comment Robot\n\n";
				$body .= "PS.  You can turn these emails off here:\n\n";
				$body .= $parent->creator->getUrl(array('page' => 'emailprefs'), null, true);
				
				$html  = $this->creator->getName() . " has posted a new reply to one of your $this->class comments.<br/><br/>";
				$html .= $parent->creator->getTicket(array(
					'module' => $this->class,
					'page' => 'view',
					'id' => $content->id
				), 'You can view it here') . ".<br/><br/>";
				$html .= "Thank you,<br/>The Happy MyKin Comment Robot<br/><br/>";
				$html .= "PS.  You can turn these emails off " . $parent->creator->getLink('.emailprefs', 'here', null, true);
				
				$parent->creator->mail($subject, $body, $html);
			}
		}
	}

	/**
	* this function generates the rss data for the comment.
	*
	* you probably wont have to define this at all.  this means AUTOMAGIC
	* comment rss feeds!!
	*
	* @return a rss item object as per BaseObject
	*/
	function getRssItem()
	{
		$rss = parent::getRssItem();

		if ($this->hasField('user_id'))
			$rss->title = 'Comment by ' . $this->creator->getName();
		else
			$rss->title = 'Comment';
		
		$rss->description = $this->body;

		return $rss;
	}

	/**
	* draw the comment.
	*
	* it is called from drawChildComments.  it draws its comment, then recurses
	* back to drawChildComments with the kids array and its id.
	*
	* @param $kids is an array of comments to use.
	*/
	function drawComment($kids = array())
	{
		global $me;
	
		if ($this->canView())
		{
			echo "<li class=\"threaded\">\n";
		
			//our title bar.
			$this->drawTitleBar();
			$this->drawCommentDetails();
		
			//draw our container
			echo "<div class=\"childContainer\">\n";
		
			//our actual content
			$this->drawCommentBody();
			$this->drawCommentReplyForm();
		
			//draw our children
			$this->drawChildComments($kids, $this->id);
			
			echo "</div>";
			
			//end comment
			echo "</li>";
		}
	}
	
	/**
	* draws the title bar for the comment
	*/
	protected function drawTitleBar()
	{
		//heres our title bar
		echo "<div class=\"titlebar\">";
		
		//our title icon
		echo '<table class="struct"><tr><td class="titleLeft">';
		echo $this->creator->getIcon('tiny');
		
		//our title div
		echo $this->creator->getName(true) . " said: ";

		echo Time::formatRelative($this->add_date);

		//mid-structure
		echo '</td><td class="titleRight">';
	
		//our options div
		echo "\t<span class=\"titleLink\" id=\"commentOptions_$this->id\">";
		echo "<a href=\"#\" onclick=\"return showCommentOptions('$this->id');\">options</a>";
		echo "</span>\n";
		echo "\t<span class=\"titleLink\" id=\"commentReplyLink_$this->id\"><a href=\"" . $this->getUrl(array('page' => 'edit', 'parent_id' => $this->id));
		echo "\" onclick=\"return showCommentReplyForm('$this->id');\">reply</a></span>\n";

		//end our stupid structure
		echo "</td></tr></table>";

		//end title bar
		echo "</div>\n";
	}
	
	/**
	* draws the details div for the comment
	*/
	protected function drawCommentDetails()
	{
		//our hidden detail div.
		echo "<div class=\"commentDetail\" id=\"comment_$this->id\" style=\"display: none;\">";
		echo "Posted by: " . $this->creator->getName(true) . "<br/>\n";
		echo "Posted: " . Time::formatRelative($this->add_date) . "<br/>\n";
		if ($this->edit_date != $this->add_date)
			echo "Last Edited on: " . Time::formatRelative($this->edit_date) . "<br/>\n";
		if ($this->canEdit())
			echo $this->getLink(".edit?id=$this->id", 'Edit Comment') . "<br/>";
		if ($this->canDelete())
			echo $this->getLink(".delete?id=$this->id", 'Delete Comment') . "<br/>";
		echo "</div>\n";
	}
	
	/**
	* draws the body of the comment w/ signature
	*/
	protected function drawCommentBody()
	{
		echo "<div class=\"commentBody\">";
		echo Linkify::bbcode($this->body);
		$this->drawCommentSignature();
		echo "</div>";
	}

	/**
	* draws a signature... you must define to tell it how.
	*/
	protected function drawCommentSignature()
	{
	}

	/**
	* draw our comment reply form.
	*/
	protected function drawCommentReplyForm()
	{
		//and our actual reply form
		echo "<div class=\"commentReply\" id=\"commentReply_$this->id\" style=\"display: none;\">\n";
		$form = $this->createReplyForm();
		$form->drawAll();
		echo "</div>\n";
	}

	/**
	* this function draws the child comments for a parent id.  usually called from a BaseThread
	*
	* @param $coms an array of comment objects to use to draw from.
	* @param $parent the id of the parent comment to draw from.
	*/
	function drawChildComments($coms, $parent)
	{
		//did we get any comments?
		if (count($coms))
		{
			if ($parent)
				echo "<div class=\"threaded\">";
				
			echo "<ol class=\"threaded\">\n";
			//oh yeah, draw them.
			foreach ($coms AS $com)
				//if they match our parent id.
				if ($com->parent_id == $parent)
					$com->drawComment($coms);
			echo "</ol>\n";

			if ($parent)
				echo "</div>";
		}
	}

	/**
	* Draws teh view page.  it redirects to the view page of the object the comment was on.
	*/
	function drawViewPage()
	{
		Util::redirect($this->getUrl("$this->class.view?id=$this->content_id"));
	}

	/**
	* gets the public data of the object... adds in the body of the comment.
	*
	* @return an array of public data for json, php serialize, etc.
	*/
	function getPublicData()
	{
		$data = parent::getPublicData();

		$data['body'] = $this->body;

		return $data;
	}

	/**
	* gets an array of fields.  see BaseObject
	*
	* @return array of fields.
	*/
	public function getCreateFieldsArray()
	{
		$fields = parent::getCreateFieldsArray();
		
		$fields['content_id'] = "content_id INT(11) default 0 not null";
		$fields['parent_id'] = "parent_id INT(11) default 0 not null";
		$fields['body'] = "body TEXT default '' not null";
	
		return $fields;
	}
	
	/**
	* gets an array of indexes.  see BaseObject
	*
	* @return array of indexes.
	*/
	public function getCreateIndexesArray()
	{
		$fields = parent::getCreateIndexesArray();
		
		$fields['content_id'] .= "KEY (content_id)";
		$fields['parent_id'] .= "KEY (parent_id)";
	
		return $fields;
	}

	/**
	* save our comment, also save parent to update comment count
	*/
	public function save()
	{
		parent::save();
		
		$obj = new $this->class($this->content_id);
		$obj->save();
	}
}
?>
