<?
/**
* @ingroup BaseModules
* @brief Pages that can be used as cron jobs.
*
* Provides ip based restrictions, defaults to blank template, provides hooks to email admins.
*/
class BaseCrons extends MyModule
{
	/**
	* preps all pages for crons
	*
	* sets template to blank, makes sure you're an admin and/or from the right IP address ( as per 'cron_request_ip' config value)
	* @see BaseModule::init
	*/
	public function init($page = null, $vars = null)
 	{
		global $me;
		
		$this->setTemplate(new BlankTemplate());

		//admins can run crons manually...
		if ($me->isAdmin() || $_SERVER["REMOTE_ADDR"] == Config::get('cron_request_ip'))
			parent::init($page, $vars);
		else
			throw new PageError("You cannot run crons, sorry $_SERVER[REMOTE_ADDR].");
	}
	
	/**
	* draws our page w/ title and status.  very basic.
	*/
	public function draw()
	{
		echo $this->pageTitle . "\n\n";
		parent::draw();
		echo "\n\nCron run!";
	}

	/**
	* attempts to get a list of admins by directly hitting users table.
	*
	* you may want to override this with more specific behaviour for your site.
	*
	* @return $an array of Users that are admins
	*/
	protected function getAdmins()
	{
		$adminRs = dbQuery("
			SELECT *
			FROM users
			WHERE is_admin = 1
		");
		while ($adminAr = dbFetchAssoc($adminRs))
			$admins[] = new User($adminAr);
		
		return $admins;
	}

	/**
	* email a message to all the admins
	*
	* @param $subject the subject of the message
	* @param $body the body of the message
	*/
	public function emailAdmins($subject, $body)
	{
		$admins = $this->getAdmins();
		if (count($admins))
			foreach ($admins AS $admin)
				$admin->mail($subject, $body);
	}
}
?>
