<?
/**
* @ingroup ObjectManagers
* @brief Get a users inbox messages
*
* Built-in privacy for only selecting the current users messages. Also bulk
* delete, and read/unread css.
*/
class BaseInbox extends MyManager
{
	/**
	* constructor
	*
	* @see ObjectManager::__construct()
	*/
	public function __construct($class = 'Message')
	{
		parent::__construct($class);
	}

	/**
	* prep page: assert login, etc
	*/
	public function initSearchPage()
	{
		$this->assertLogin();
		
		parent::initSearchPage();

		$this->pageTitle = 'Inbox';
	}

	/**
	* add extra param to search page:  read status
	*
	* @see ObjectManager::getSearchParamXml()
	*/
	public function getSearchParamXml()
	{ 
		$xml = parent::getSearchParamXml();

		$xml .= <<<XML
			<param name="status">
				<option value="read"/>
				<option value="unread"/>
			</param>
XML;
		return $xml;
	}

	/**
	* add read status to select
	*
	* @see ObjectManager::getSearchSelect()
	*/
	protected function getSearchSelect($params)
	{ 
		$sql = parent::getSearchSelect($params);
		
		$sql .= ", i.read ";

		return $sql;
	}

	/**
	* join us on the inbox table
	*
	* @see ObjectManager::getSearchFrom()
	*/	
	protected function getSearchFrom($params)
	{
		$from = "FROM {$this->object->inboxTable} i\n";
		$from .= "INNER JOIN {$this->object->tableName} o\n";
		$from .= "ON o.id = i.message_id\n";

		return $from;
	}

	/**
	* get where SQL: user privacy, read status, etc
	*
	* @see ObjectManager::getSearchWhere()
	*/
	protected function getSearchWhere($params)
	{
		global $me;
		
		$where = parent::getSearchWhere($params);
		
		$where .= " AND i.user_id = '$me->id'\n";

		if ($params['status'] == 'unread')
			$where .= " AND i.read = 0 ";
		
		if ($params['status'] == 'read')
			$where .= " AND i.read = 1 ";

		return $where;
	}

	/**
	* add read status as sort option
	* @see ObjectManager::getSortFieldsArray()
	*/
	protected function getSortFieldsArray()
	{
		$data = parent::getSortFieldsArray();

		$data['read'] = 'Sort by read status';

		return $data;
	}

	/**
	* load up our object data
	* @param $data the search data array from mysql
	* @return an array of BaseMessage objects
	*/
	protected function loadObjects($data)
	{
		$objs = parent::loadObjects($data);

		foreach ($data AS $row)
			$objs[$row['id']]->read = $row['read'];

		return $objs;
	}

	/**
	* draw our rows
	*
	* add header row, checkboxes, delete form, etc.
	*
	* @param $rs array of BaseMessage objects to draw
	*/
	public function drawRows($rs)
	{
		if (count($rs))
		{
			echo "<form id=\"deleteAll\" method=\"post\" action=\"" . $this->getUrl(array('page' => 'delete')) . "\">\n";
			
			echo "<table class=\"InOutBox\">";
			$this->object->drawHeaderRow('inbox');
			foreach ($rs AS $ar)
			{
				$on = !$on;
				$ar->drawRow('inbox', $on);
			}
			echo "</table>";

			echo "<input type=\"submit\" name=\"submit\" value=\"Delete Selected\"/>\n";
			echo "</form>\n";
		}
	}

	/**
	* get extra pages
	*
	* delete - bulk delete inbox messages
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= "<page name=\"delete\" description=\"delete inbox messages.\"/>\n";

		return $xml;
	}

	/**
	* prep our delete page
	*/
	public function initDeletePage()
	{
		global $me;
		
		$this->pageTitle = 'Delete Inbox Messages';

		$this->assertLogin();
		
		if (!count($_POST['message']))
			throw new PageError('You must select at least 1 message to delete.');
		else
		{
			foreach ($_POST['message'] AS $messageId)
			{
				dbExecute("
					DELETE FROM {$this->object->inboxTable}
					WHERE user_id = '$me->id'
						AND message_id = '$messageId'
				");
			}

			Util::redirect($this->getUrl(".main"));
		}
	}

	/**
	* give a message back about the delete status
	*/
	public function drawDeletePage()
	{
		echo 'Messages deleted.';
	}
}
?>
