<?
/**
* @ingroup BaseModules
* @brief A nice easy class for doing Google Maps Mashups.
*
* A class that provides a really nice base for Google Maps Mashups.  Draws teh
* map, provides a geocoding jumper javascript, and lots of other cool stuff
* like clustered markers, and nice markers with labels.
*/
abstract class BaseMap extends MyModule
{
	/**
	* adds the lookup page and the mapjs page.
	*/
	function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= <<<XML
			<page name="lookup">
				<param name="location" required="true"/>
			</page>
			<page name="mapjs"/>
XML;
		return $xml;
	}

	/**
	* intializes the main page.
	*
	* adds some needed files like the google map file, prototype.js and your
	* map javascript page.
	*/
	function initMainPage()
	{
		$this->needsJs("http://maps.google.com/maps?file=api&v=2&key=" . Config::get('google_map_api_key'));
		$this->needsJs("lib/js/prototype-1.5.0.js");
		$this->needsJs("lib/js/gxmarker.js");
		
		$this->needsJs($this->getUrl('.mapjs'));

		$this->doCachePage = true;
	}

	/**
	* draws the map and a div for debug/status messages.
	*/
	function drawMainPage()
	{
		$this->drawLookupForm();
		echo "<div id=\"mapDebug\"></div>\n";
		echo "<div id=\"map\"></div>\n";
	}

	/**
	* draws a form for users to jump to a certain location... super handy!!!
	*/
	function drawLookupForm()
	{
?>
		<div id="mapLookup">
			<form id="mapLookup" onsubmit="mapDoLookup(); return false;">
				<b>Jump to location: (street/city/state/zip):</b>
				<input type="text" size="40" id="lookupAddress"/>
				<input type="button" onclick="mapDoLookup()" value="Jump"/>
			</form>
		</div>
<?
	}

	/**
	* inits our lookup page... no header/footer
	*/
	function initLookupPage()
	{
		$this->setTemplate(new BlankTemplate());
	}

	/**
	* draws our lookup page... geocoded JSON export baby!
	*/
	function drawLookupPage()
	{
		$geo = new Geocode();
		$rs = $geo->search($this->params('location'));

		echo JSON::encode($rs);
	}

	/**
	* inits our javascript page... javascript content type and no header/footer
	* action
	*/
	function initMapJsPage()
	{
		$this->setTemplate(new BlankTemplate());
		header("Content-Type: application/x-javascript");
	}

	/**
	* draws our map javascript page.
	*
	* provides a maplookup function to handle lookups, also a mapdebug function
	* to give messages to user. extend this to add your customized map js
	* stuff.  its simply a javascript file.
	*/
	function drawMapJsPage()
	{
?>
function mapDoLookup()
{
	var addy = $F('lookupAddress');

	if (addy)
	{
		Form.disable('mapLookup');
		
		var url = '<?=$this->getUrl('.lookup?location=');?>' + addy;
		
		GDownloadUrl(url, function(data, response)
		{
			var result;
			eval('result = ' + data);

			if (result)
			{
				var lon = result.lon;
				var lat = result.lat;

				map.panTo(new GLatLng(lat, lon));
				
				$('mapDebug').innerHTML = '';
			}
			else
				$('mapDebug').innerHTML = '<b>Location not found.</b>';

			Form.enable('mapLookup');
		});
	}
}

function mapDebug(txt)
{
	$('mapDebug').innerHTML += txt + '<br/>';
}
<?
	} 
}
?>
