<?
/**
* @ingroup ObjectManagers
* @brief Get a users outbox messages
*
* Built-in privacy for only selecting the current users messages. Also bulk
* delete, and read/unread css.
*/
class BaseOutbox extends MyManager
{
	/**
	* constructor
	*
	* @see ObjectManager::__construct()
	*/
	public function __construct($class = 'Message')
	{
		parent::__construct($class);
	}
	
	/**
	* prep page: assert login, etc
	*/
	public function initSearchPage()
	{
		$this->assertLogin();
		
		parent::initSearchPage();

		$this->pageTitle = 'Outbox';
	}
	
	/**
	* join us on the outbox table
	*
	* @see ObjectManager::getSearchFrom()
	*/	
	protected function getSearchFrom($params)
	{
		$from = "FROM {$this->object->outboxTable} b\n";
		$from .= "INNER JOIN {$this->object->tableName} o\n";
		$from .= "ON o.id = b.message_id\n";

		return $from;
	}
	
	/**
	* get where SQL: make sure its only our messages
	*
	* @see ObjectManager::getSearchWhere()
	*/
	protected function getSearchWhere($params)
	{
		global $me;

		$where = parent::getSearchWhere($params);
		$where .= " AND b.user_id = '$me->id'\n";

		return $where;
	}
	
	/**
	* draw our rows
	*
	* add header row, checkboxes, delete form, etc.
	*
	* @param $rs array of BaseMessage objects to draw
	*/
	public function drawRows($rs)
	{
		if (count($rs))
		{
			echo "<form id=\"deleteAll\" method=\"post\" action=\"" . $this->getUrl(array('page' => 'delete')) . "\">\n";
			echo "<table class=\"InOutBox\">";
			$this->object->drawHeaderRow('outbox');
			foreach ($rs AS $ob)
			{
				$on = !$on;
				$ob->drawRow('outbox', $on);
			}
			echo "</table>";

			echo "<input type=\"submit\" name=\"submit\" value=\"Delete Selected\"/>\n";
			echo "</form>\n";
		}
	}
	
	/**
	* get extra pages
	*
	* delete - bulk delete outbox messages
	*/
	public function getPagesXml()
	{
		$xml = parent::getPagesXml();

		$xml .= "<page name=\"delete\" description=\"delete outbox messages.\"/>\n";

		return $xml;
	}

	/**
	* prep our delete page
	*/
	public function initDeletePage()
	{
		global $me;
		
		$this->pageTitle = 'Delete Outbox Messages';

		$this->assertLogin();
		
		if (!count($_POST['message']))
			throw new PageError('You must select at least 1 message to delete.');
		else
		{
			foreach ($_POST['message'] AS $messageId)
			{
				dbExecute("
					DELETE FROM {$this->object->outboxTable}
					WHERE user_id = '$me->id'
						AND message_id = '$messageId'
				");
			}

			Util::redirect($this->getUrl(".main"));
		}
	}

	/**
	* give a message back about the delete status
	*/
	public function drawDeletePage()
	{
		echo 'Messages deleted.';
	}
}
?>
