<?
/**
* @defgroup SMS Simple Message Service
* @ingroup BJHelpers
* @brief You can easily send short messages to mobile phones.
*
* @{
*/

/**
* @brief SMS is a class for easy sending of SMS messages.
*/
class SMS
{
	/**
	* the 10 digit number you are sending the text from.
	*/
	public $toNumber = null;

	/**
	* the 10 digit number you are sending the text to.
	*/
	public $fromNumber = null;

	/**
	* an array of SMSProviders
	*/
	public $providers;

	/**
	* setup our SMS class.
	*
	* Create providers array, etc.
	*/
	public function __construct()
	{ 
		$this->providers = array(
			'alltel' => new EmailSMSProvider('Cingular', '###@message.alltel.com'),
			'cingular' => new EmailSMSProvider('Cingular', '###@cingularME.com'),
			'nextel' => new EmailSMSProvider('Nextel', '###@messaging.nextel.com'),
			'sprint' => new EmailSMSProvider('Sprint', '###@messaging.sprintpcs.com'),
			'tmobile' => new EmailSMSProvider('T-Mobile', '###@@tmomail.net'),
			'uscellular' => new EmailSMSProvider('US Cellular', '###@email.uscc.net'),
			'verizon' => new EmailSMSProvider('Verizon', '###@vtext.com'),
			'virgin' => new EmailSMSProvider('Virgin Mobile', '###@vmobl.com'),
		);

		//services we need to test...
		//alltel
		//nextel
		//sprint
		//tmobile
		//virgin-mobile
	}

	/**
	* setup our to number.
	*
	* @param $number a 10 digit phone number
	* @param $provider a string that matches the a key in the providers array
	*/
	public function setTo($number, $provider)
	{
		$this->toNumber = $this->getProvider($number, $provider);
	}

	/**
	* setup our from number
	*
	* @param $number a 10 digit phone number
	* @param $provider a string that matches the a key in the providers array
	*/
	public function setFrom($number, $provider)
	{
		$this->fromNumber = $this->getProvider($number, $provider);
	}

	/**
	* send the actual SMS message
	*
	* @param $message our message to send.  warning: must be <= 160 characters.
	* @param $from an optional from email, if not sending from another cell number.
	*/
	public function send($message, $from = null)
	{
		//did we get a from?
		if ($from === null && $this->FromNumber instanceOf SMSProvider)
			$from = $this->fromNumber->getEmail();

		//if its still blank... set our default!
		if ($from == '')
		{
			if (Config::get('sms_from_email'))
				$from = Config::get('sms_from_email');
			else if (Config::get('mailer_from'))
				$from = Config::get('mailer_from');
			else
				throw new Exception("Undefined from email.");
		}
			
		//send it off!
		$this->toNumber->send($message, $from);
	}

	/**
	* get the provider from the array of providers.
	*
	* @param $number the 10 digit phone number
	* @param $provider a string name of the provider that matches a key in the providers array
	*/
	public function getProvider($number, $provider)
	{
		if ($this->providers[$provider] instanceOf SMSProvider)
		{
			$provider = $this->providers[$provider];
			$provider->prepNumber($number);
			
			return $provider;
		}
		else
			throw new Exception("Provider '$provider' not found.");
	}
}

/**
* @brief Send SMS's to a provider.
*/
abstract class SMSProvider
{
	/**
	* the name of the provider.
	*/
	public $name;

	/**
	* the 10 digit number
	*/
	public $number;
	
	/**
	* instantiate our class
	*
	* @param $name the name of the provider
	*/
	public function __construct($name)
	{
		$this->name = $name;
	}

	/**
	* correctly format a 10 digit number
	*
	* @param $number the number string to format.
	* @return the properly formatted number
	*/
	public static function formatNumber($number)
	{
		return preg_replace("/[^0-9]*/", "", $number);
	}

	/**
	* prepare the number for sending
	*
	* @param $number the number to prepare.
	*/
	public function prepNumber($number)
	{
		//format number..
		$number = self::formatNumber($number);
		
		//save
		$this->number = $number;
	}

	/**
	* send the sms message
	*
	* @param $message the message to send (<160 chars)
	* @param $from the email to send it from.
	*/
	abstract public function send($message, $from);
}

/**
* @brief Send SMS via email gateway.
*
* Currently the only way to send SMS messages with BaseJumper.  Uses the providers email gateway... dont abuse it.
*/
class EmailSMSProvider extends SMSProvider
{
	/**
	* the email gateway address
	*/
	private $email;
	
	/**
	* create our email provider.
	*
	* @param $name the name of the provider
	* @param $email the email of the provider gateway, with ### where the 10 digit number will go.
	*/
	public function __construct($name, $email)
	{
		parent::__construct($name);

		$this->email = $email;
	}

	/**
	* get the email address to send to.
	*
	* @return the email for our specific number / provider combo.
	*/
	public function getEmail()
	{
		return str_replace("###", $this->number, $this->email);
	}

	/**
	* send a sms message to a gateway.
	*
	* @param $message the text message (<160 chars)
	* @param $from the from email address
	*/
	public function send($message, $from)
	{
		//put our number in.
		$email = $this->getEmail();
		
		//where does it go?
		Mail::send($email, '', $message, '', $from);
	}
}

/*@}*/
?>
