<?
/**
* @ingroup BJTemplate
* @brief The mother template.
*
* It handles lots of good stuff from output buffering to getting errors and such.
*/
abstract class BaseTemplate
{
	public $object;

	public $content = null;

	/**
	 * This function draws the entire page.  It draws the header/footer if
	 * needed, and then has the object draw the actual content.
	 */
	public function draw()
	{
		global $sqlQueryCount;
		
		//load our page to get stuff like js/css/feeds
		if (!$this->object->hasError())
			$this->getContent();

		//draw our header
		$this->drawHeader($this->object->getTitle());
		
		//obviously want to draw the objects page
		$this->drawPage();

		//and we should draw the footer too
		$this->drawFooter();
	}

	/**
	 * This function draws the header of the page.
	 * @param $title is the title of the page
	 */
	public function drawHeader($title)
	{
	}

	/**
	 * This function draws the content area of the page.
	 */
	public function drawPage()
	{
		//draw our errors...
		if ($this->object->hasError())
			$this->drawError();
		//nope, we're clean.
		else
		{
			//do we draw status?
			$this->drawStatus();
		
			//do we draw content?
			$this->drawContent();
		}
	}

	/**
	* This draws all the errors that the page init generated.  they are all
	* wrapped in a <p> with class pf 'baseError' and an id of 'error-$key'
	* where key is the key in the error array.
	*/
	public function drawError()
	{
		//draw our errors!
		$error = $this->object->getError();
		echo "<p id=\"error-$key\" class=\"BaseError\">$error</p>\n";
	}

	/**
	* This draws all the status messages that the page init generated.  they
	* are all wrapped in a <p> with class pf 'baseStatus' and an id of
	* 'status-$key' where key is the key in the status array.
	*/
	public function drawStatus()
	{
		$status = $this->object->getStatus();
		echo implode("\r\n", $status);
	}

	/**
	* This draws teh content by calling draw() on the object in question.
	*/
	public function drawContent()
	{
		echo $this->getContent();
	}

	/**
	* this function gets our content for display.
	*
	* @return the content string.
	*/
	public function getContent()
	{
		//if its not already set.. get it.
		if ($this->content === null)
			$this->content = $this->object->getContent();
		
		return $this->content;
	}

	/**
	 * This function draws teh footer of the page.
	 */
	public function drawFooter()
	{
	}

	/**
	* this function draws text that gets replaced with page generation stats.
	*/
	public function drawGenerationStats()
	{
		echo "Generated in %%BJ_RUNTIME%% seconds. %%BJ_QUERIES%% db queries. %%BJ_CACHE_HITS%% cache hits.";
	}

	/**
	* this function draws a link to recache the page (if it needs it...) this
	* is only for entire page caches, and not object caches.
	*
	* @param $link the text to link
	*/
	public function drawRecachePageLink($link = 'Recache Page')
	{
		if ($this->object->doCachePage)
			echo $this->object->getLink("main.recache?do=" . base64_encode($_SERVER['REQUEST_URI']), $link);
	}
}
?>
