<?
/**
* @ingroup BJHelpers
* @brief Time is a class for formatting of time.
*
* Right now it only handles relative time display.
*/
class Time
{
	/**
	* format a time relative to now
	*
	* @param $startTime a timestamp of some form to use as the base time
	* @param $parens do we wrap it in parenthesis or not?
	* @return a relatively formated time string.  the full date is given too as an <acronym>
	*/
	public static function formatRelative($startTime, $parens = true)
	{
		global $me;
		
		//make our start and end time.
		$endTime = time();
		$formatStart = $me->formatDateTime($startTime);
		$startTime = strtotime($startTime);
		
		if ($startTime > $endTime)
		{
			$time = Time::elapsed($endTime, $startTime);
			$time = "in " . $time;
		}
		else
		{
			$time = Time::elapsed($startTime, $endTime);
			$time .= " ago";
		}

		if ($parens)
			return "<acronym class=\"relativeTime\" title=\"$formatStart\">($time)</acronym>";
		else
			return "<acronym class=\"relativeTime\" title=\"$formatStart\">$time</acronym>";
	}

	/**
	* get the elapses time since (or until) a date
	* @param $startTime the time to start (in seconds since epoch)
	* @param $endTime th etime to end (in seconds since epoch)
	* @param $detail do we want a detailed time: years, months, weeks, days, etc;  OR just the largest time (1 month, or 1 day, etc.)
	* @return the elapsed time string.
	*/
	 public static function elapsed($startTime, $endTime, $detail = false)
	 {
		//how long?
		$elapsed = $endTime - $startTime;

		//how many years?
		$years = round($elapsed / 31556926);
		if ($years && !$detail)
			return "$years " . Util::pluralize('year', $years);
		$elapsed = $elapsed - ($years * 31556926);
		
		//okay, months?
		$months = round($elapsed / 2629743.83);
		if ($months && !$detail)
			return "$months " . Util::pluralize('month', $months);
		$elapsed = $elapsed - ($months * 2629743.83);

		//weeks?
		$weeks = round($elapsed / 604800);
		if ($weeks && !$detail)
			return "$weeks " . Util::pluralize('week', $weeks);
		$elapsed = $elapsed - ($weeks * 604800);

		//how bout days?
		$days = round($elapsed / 86400);
		if ($days && !$detail)
			return "$days " . Util::pluralize('day', $days);
		$elapsed = $elapsed - ($days * 86400);
		
		//how bout hours?
		$hours = round($elapsed / 3600);
		if ($hours && !$detail)
			return "$hours " . Util::pluralize('hour', $hours);
		$elapsed = $elapsed - ($hours * 3600);
		
		//how bout minutes?
		$minutes = round($elapsed / 60);
		if ($minutes && !$detail)
			return "$minutes " . Util::pluralize('minute', $minutes);
		$elapsed = $elapsed - ($minutes * 60);

		//finally, seconds...
		$seconds = $elapsed;
		if (!$detail && $seconds)
			return "$seconds " . Util::pluralize('second', $seconds);

		if (!$detail)
			return "now";

		//if we havent already returned
		return "$years years, $months months, $weeks weeks, $days days, $hours hours, $minutes minutes, and $seconds seconds ";
	 }
}
?>
